@@ -17,6 +17,7 @@ package groovy.lang;
 
 import org.codehaus.groovy.runtime.metaclass.MetaClassRegistryImpl;
 import org.codehaus.groovy.util.ReferenceBundle;
+import org.codehaus.groovy.util.ReleaseInfo;
 
 public final class GroovySystem {
     //
@@ -78,4 +79,11 @@ public final class GroovySystem {
         ReferenceBundle.getSoftBundle().getManager().stopThread();
         ReferenceBundle.getWeakBundle().getManager().stopThread();
     }
+
+    /**
+     * Returns the groovy version
+     */
+    public static String getVersion() {
+        return ReleaseInfo.getVersion();
+    }
 }
