@@ -23,7 +23,6 @@ import org.codehaus.groovy.ast.CodeVisitorSupport;
 import org.codehaus.groovy.ast.stmt.Statement;
 
 import java.sql.Connection;
-import java.sql.PreparedStatement;
 import java.sql.SQLException;
 import java.sql.ResultSet;
 import java.util.ArrayList;
@@ -37,7 +36,7 @@ import java.util.logging.Level;
  * @author Chris Stevenson
  * @author Paul King
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 16580 $
+ * @version $Revision: 18226 $
  */
 public class DataSet extends Sql {
 
@@ -149,25 +148,9 @@ public class DataSet extends Sql {
         buffer.append(paramBuffer.toString());
         buffer.append(")");
 
-        Connection connection = createConnection();
-        PreparedStatement statement = null;
-        try {
-            statement = connection.prepareStatement(buffer.toString());
-            int i = 1;
-            for (Object value : map.values()) {
-                setObject(statement, i++, value);
-            }
-            int answer = statement.executeUpdate();
-            if (answer != 1) {
-                log.log(Level.WARNING, "Should have updated 1 row not " + answer + " when trying to add: " + map);
-            }
-        }
-        catch (SQLException e) {
-            log.log(Level.WARNING, "Failed to add row for: " + map, e);
-            throw e;
-        }
-        finally {
-            closeResources(connection, statement);
+        int answer = executeUpdate(buffer.toString(), new ArrayList<Object>(map.values()));
+        if (answer != 1) {
+            log.log(Level.WARNING, "Should have updated 1 row not " + answer + " when trying to add: " + map);
         }
     }
 
