@@ -22,6 +22,11 @@ import java.util.Date;
 import java.util.List;
 import java.util.ArrayList;
 
+/**
+ * Base class for date and time durations.
+ * @see Duration
+ * @author John Wilson tug@wilson.co.uk
+ */
 public abstract class BaseDuration {
     protected final int years;
     protected final int months;    
@@ -100,7 +105,11 @@ public abstract class BaseDuration {
         if (this.seconds != 0 || this.millis != 0)
             buffer.add(this.seconds + "." + DefaultGroovyMethods.padLeft("" + this.millis, new Integer(3), "0")  + " seconds");
 
-        return DefaultGroovyMethods.join(buffer, ", ");
+        if (buffer.size()!=0) {
+            return DefaultGroovyMethods.join(buffer, ", ");
+        } else {
+            return "0";
+        }
     }
 
     public abstract long toMilliseconds();
