@@ -1,4 +1,4 @@
-// $ANTLR 2.7.6 (2005-12-22): "groovy.g" -> "GroovyLexer.java"$
+// $ANTLR 2.7.7 (20060906): "groovy.g" -> "GroovyLexer.java"$
 
 package org.codehaus.groovy.antlr.parser;
 import org.codehaus.groovy.antlr.*;
@@ -8,6 +8,8 @@ import java.io.Reader;
 import antlr.InputBuffer;
 import antlr.LexerSharedInputState;
 import antlr.CommonToken;
+import org.codehaus.groovy.GroovyBugError;
+import antlr.TokenStreamRecognitionException;
 
 import java.io.InputStream;
 import antlr.TokenStreamException;
@@ -72,11 +74,23 @@ public class GroovyLexer extends antlr.CharScanner implements GroovyTokenTypes,
     protected ArrayList parenLevelStack = new ArrayList();
     protected int lastSigTokenType = EOF;  // last returned non-whitespace token
 
+    public void setTokenObjectClass(String name) {/*ignore*/}
+    
+    protected Token makeToken(int t) {
+        GroovySourceToken tok = new GroovySourceToken(t);
+        tok.setColumn(inputState.getTokenStartColumn());
+        tok.setLine(inputState.getTokenStartLine());
+        tok.setColumnLast(inputState.getColumn());
+        tok.setLineLast(inputState.getLine());
+        return tok;
+    }
+    
     protected void pushParenLevel() {
-        parenLevelStack.add(new Integer(parenLevel*SCS_LIMIT + stringCtorState));
+        parenLevelStack.add(Integer.valueOf(parenLevel*SCS_LIMIT + stringCtorState));
         parenLevel = 0;
         stringCtorState = 0;
     }
+    
     protected void popParenLevel() {
         int npl = parenLevelStack.size();
         if (npl == 0)  return;
@@ -273,7 +287,7 @@ public class GroovyLexer extends antlr.CharScanner implements GroovyTokenTypes,
             }
             ttypes = map;
         }
-        Integer tt = new Integer(t.getType());
+        Integer tt = Integer.valueOf(t.getType());
         Object ttn = ttypes.get(tt);
         if (ttn == null)  ttn = "<"+tt+">";
         return "["+ttn+",\""+t.getText()+"\"]";
@@ -304,8 +318,8 @@ public GroovyLexer(LexerSharedInputState state) {
 	literals.put(new ANTLRHashString("short", this), new Integer(104));
 	literals.put(new ANTLRHashString("break", this), new Integer(140));
 	literals.put(new ANTLRHashString("while", this), new Integer(135));
-	literals.put(new ANTLRHashString("new", this), new Integer(190));
-	literals.put(new ANTLRHashString("instanceof", this), new Integer(177));
+	literals.put(new ANTLRHashString("new", this), new Integer(155));
+	literals.put(new ANTLRHashString("instanceof", this), new Integer(154));
 	literals.put(new ANTLRHashString("implements", this), new Integer(128));
 	literals.put(new ANTLRHashString("synchronized", this), new Integer(117));
 	literals.put(new ANTLRHashString("const", this), new Integer(40));
@@ -313,7 +327,7 @@ public GroovyLexer(LexerSharedInputState state) {
 	literals.put(new ANTLRHashString("package", this), new Integer(78));
 	literals.put(new ANTLRHashString("return", this), new Integer(139));
 	literals.put(new ANTLRHashString("throw", this), new Integer(142));
-	literals.put(new ANTLRHashString("null", this), new Integer(193));
+	literals.put(new ANTLRHashString("null", this), new Integer(156));
 	literals.put(new ANTLRHashString("def", this), new Integer(81));
 	literals.put(new ANTLRHashString("threadsafe", this), new Integer(116));
 	literals.put(new ANTLRHashString("protected", this), new Integer(113));
@@ -342,7 +356,7 @@ public GroovyLexer(LexerSharedInputState state) {
 	literals.put(new ANTLRHashString("char", this), new Integer(103));
 	literals.put(new ANTLRHashString("private", this), new Integer(111));
 	literals.put(new ANTLRHashString("default", this), new Integer(126));
-	literals.put(new ANTLRHashString("false", this), new Integer(192));
+	literals.put(new ANTLRHashString("false", this), new Integer(153));
 	literals.put(new ANTLRHashString("this", this), new Integer(129));
 	literals.put(new ANTLRHashString("static", this), new Integer(80));
 	literals.put(new ANTLRHashString("abstract", this), new Integer(38));
@@ -353,7 +367,7 @@ public GroovyLexer(LexerSharedInputState state) {
 	literals.put(new ANTLRHashString("in", this), new Integer(138));
 	literals.put(new ANTLRHashString("void", this), new Integer(100));
 	literals.put(new ANTLRHashString("switch", this), new Integer(136));
-	literals.put(new ANTLRHashString("true", this), new Integer(191));
+	literals.put(new ANTLRHashString("true", this), new Integer(157));
 	literals.put(new ANTLRHashString("long", this), new Integer(107));
 }
 
@@ -427,12 +441,6 @@ tryAgain:
 					theRetToken=_returnToken;
 					break;
 				}
-				case '$':
-				{
-					mDOLLAR(true);
-					theRetToken=_returnToken;
-					break;
-				}
 				case '\t':  case '\u000c':  case ' ':  case '\\':
 				{
 					mWS(true);
@@ -1328,7 +1336,7 @@ tryAgain:
 		_returnToken = _token;
 	}
 	
-	public final void mDOLLAR(boolean _createToken) throws RecognitionException, CharStreamException, TokenStreamException {
+	protected final void mDOLLAR(boolean _createToken) throws RecognitionException, CharStreamException, TokenStreamException {
 		int _ttype; Token _token=null; int _begin=text.length();
 		_ttype = DOLLAR;
 		int _saveIndex;
@@ -1503,8 +1511,8 @@ tryAgain:
 		int _saveIndex;
 		
 		{
-		int _cnt578=0;
-		_loop578:
+		int _cnt606=0;
+		_loop606:
 		do {
 			if ((LA(1)=='\\') && (LA(2)=='\n'||LA(2)=='\r') && (true) && (true)) {
 				match('\\');
@@ -1520,10 +1528,10 @@ tryAgain:
 				match('\f');
 			}
 			else {
-				if ( _cnt578>=1 ) { break _loop578; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+				if ( _cnt606>=1 ) { break _loop606; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 			}
 			
-			_cnt578++;
+			_cnt606++;
 		} while (true);
 		}
 		if ( inputState.guessing==0 ) {
@@ -1586,8 +1594,8 @@ tryAgain:
 		{
 		if (((LA(1)=='\t'||LA(1)=='\n'||LA(1)=='\u000c'||LA(1)=='\r'||LA(1)==' '||LA(1)=='/'||LA(1)=='\\'))&&(!whitespaceIncluded)) {
 			{
-			int _cnt584=0;
-			_loop584:
+			int _cnt612=0;
+			_loop612:
 			do {
 				switch ( LA(1)) {
 				case '\n':  case '\r':
@@ -1608,10 +1616,10 @@ tryAgain:
 						mML_COMMENT(false);
 					}
 				else {
-					if ( _cnt584>=1 ) { break _loop584; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+					if ( _cnt612>=1 ) { break _loop612; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 				}
 				}
-				_cnt584++;
+				_cnt612++;
 			} while (true);
 			}
 		}
@@ -1645,7 +1653,7 @@ tryAgain:
 		
 		match("//");
 		{
-		_loop588:
+		_loop616:
 		do {
 			if ((_tokenSet_1.member(LA(1))) && (true) && (true) && (true)) {
 				{
@@ -1653,7 +1661,7 @@ tryAgain:
 				}
 			}
 			else {
-				break _loop588;
+				break _loop616;
 			}
 			
 		} while (true);
@@ -1675,12 +1683,12 @@ tryAgain:
 		
 		match("/*");
 		{
-		_loop598:
+		_loop626:
 		do {
-			boolean synPredMatched596 = false;
-			if (((LA(1)=='*') && ((LA(2) >= '\u0003' && LA(2) <= '\ufffe')) && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe')) && (true))) {
-				int _m596 = mark();
-				synPredMatched596 = true;
+			boolean synPredMatched624 = false;
+			if (((LA(1)=='*') && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && ((LA(3) >= '\u0000' && LA(3) <= '\ufffe')) && (true))) {
+				int _m624 = mark();
+				synPredMatched624 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1689,12 +1697,12 @@ tryAgain:
 					}
 				}
 				catch (RecognitionException pe) {
-					synPredMatched596 = false;
+					synPredMatched624 = false;
 				}
-				rewind(_m596);
+				rewind(_m624);
 inputState.guessing--;
 			}
-			if ( synPredMatched596 ) {
+			if ( synPredMatched624 ) {
 				match('*');
 			}
 			else if ((LA(1)=='\n'||LA(1)=='\r')) {
@@ -1706,7 +1714,7 @@ inputState.guessing--;
 				}
 			}
 			else {
-				break _loop598;
+				break _loop626;
 			}
 			
 		} while (true);
@@ -1731,7 +1739,7 @@ inputState.guessing--;
 		  throw new SemanticException("getLine() == 1 && getColumn() == 1");
 		match("#!");
 		{
-		_loop592:
+		_loop620:
 		do {
 			if ((_tokenSet_1.member(LA(1)))) {
 				{
@@ -1739,7 +1747,7 @@ inputState.guessing--;
 				}
 			}
 			else {
-				break _loop592;
+				break _loop620;
 			}
 			
 		} while (true);
@@ -1760,10 +1768,10 @@ inputState.guessing--;
 		int _saveIndex;
 		int tt=0;
 		
-		boolean synPredMatched601 = false;
-		if (((LA(1)=='\'') && (LA(2)=='\'') && (LA(3)=='\'') && ((LA(4) >= '\u0003' && LA(4) <= '\ufffe')))) {
-			int _m601 = mark();
-			synPredMatched601 = true;
+		boolean synPredMatched629 = false;
+		if (((LA(1)=='\'') && (LA(2)=='\'') && (LA(3)=='\'') && ((LA(4) >= '\u0000' && LA(4) <= '\ufffe')))) {
+			int _m629 = mark();
+			synPredMatched629 = true;
 			inputState.guessing++;
 			try {
 				{
@@ -1771,17 +1779,17 @@ inputState.guessing--;
 				}
 			}
 			catch (RecognitionException pe) {
-				synPredMatched601 = false;
+				synPredMatched629 = false;
 			}
-			rewind(_m601);
+			rewind(_m629);
 inputState.guessing--;
 		}
-		if ( synPredMatched601 ) {
+		if ( synPredMatched629 ) {
 			_saveIndex=text.length();
 			match("'''");
 			text.setLength(_saveIndex);
 			{
-			_loop606:
+			_loop634:
 			do {
 				switch ( LA(1)) {
 				case '\\':
@@ -1805,10 +1813,10 @@ inputState.guessing--;
 					break;
 				}
 				default:
-					boolean synPredMatched605 = false;
-					if (((LA(1)=='\'') && ((LA(2) >= '\u0003' && LA(2) <= '\ufffe')) && ((LA(3) >= '\u0003' && LA(3) <= '\ufffe')) && ((LA(4) >= '\u0003' && LA(4) <= '\ufffe')))) {
-						int _m605 = mark();
-						synPredMatched605 = true;
+					boolean synPredMatched633 = false;
+					if (((LA(1)=='\'') && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && ((LA(3) >= '\u0000' && LA(3) <= '\ufffe')) && ((LA(4) >= '\u0000' && LA(4) <= '\ufffe')))) {
+						int _m633 = mark();
+						synPredMatched633 = true;
 						inputState.guessing++;
 						try {
 							{
@@ -1829,19 +1837,19 @@ inputState.guessing--;
 							}
 						}
 						catch (RecognitionException pe) {
-							synPredMatched605 = false;
+							synPredMatched633 = false;
 						}
-						rewind(_m605);
+						rewind(_m633);
 inputState.guessing--;
 					}
-					if ( synPredMatched605 ) {
+					if ( synPredMatched633 ) {
 						match('\'');
 					}
 					else if ((_tokenSet_4.member(LA(1)))) {
 						mSTRING_CH(false);
 					}
 				else {
-					break _loop606;
+					break _loop634;
 				}
 				}
 			} while (true);
@@ -1851,10 +1859,10 @@ inputState.guessing--;
 			text.setLength(_saveIndex);
 		}
 		else {
-			boolean synPredMatched610 = false;
-			if (((LA(1)=='"') && (LA(2)=='"') && (LA(3)=='"') && ((LA(4) >= '\u0003' && LA(4) <= '\ufffe')))) {
-				int _m610 = mark();
-				synPredMatched610 = true;
+			boolean synPredMatched638 = false;
+			if (((LA(1)=='"') && (LA(2)=='"') && (LA(3)=='"') && ((LA(4) >= '\u0000' && LA(4) <= '\ufffe')))) {
+				int _m638 = mark();
+				synPredMatched638 = true;
 				inputState.guessing++;
 				try {
 					{
@@ -1862,12 +1870,12 @@ inputState.guessing--;
 					}
 				}
 				catch (RecognitionException pe) {
-					synPredMatched610 = false;
+					synPredMatched638 = false;
 				}
-				rewind(_m610);
+				rewind(_m638);
 inputState.guessing--;
 			}
-			if ( synPredMatched610 ) {
+			if ( synPredMatched638 ) {
 				_saveIndex=text.length();
 				match("\"\"\"");
 				text.setLength(_saveIndex);
@@ -1884,7 +1892,7 @@ inputState.guessing--;
 					++suppressNewline;
 				}
 				{
-				_loop608:
+				_loop636:
 				do {
 					switch ( LA(1)) {
 					case '\\':
@@ -1907,7 +1915,7 @@ inputState.guessing--;
 							mSTRING_CH(false);
 						}
 					else {
-						break _loop608;
+						break _loop636;
 					}
 					}
 				} while (true);
@@ -1919,7 +1927,7 @@ inputState.guessing--;
 				match('\'');
 				text.setLength(_saveIndex);
 			}
-			else if ((LA(1)=='"') && ((LA(2) >= '\u0003' && LA(2) <= '\ufffe')) && (true) && (true)) {
+			else if ((LA(1)=='"') && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && (true) && (true)) {
 				_saveIndex=text.length();
 				match('"');
 				text.setLength(_saveIndex);
@@ -2031,17 +2039,17 @@ inputState.guessing--;
 			case 'u':
 			{
 				{
-				int _cnt635=0;
-				_loop635:
+				int _cnt663=0;
+				_loop663:
 				do {
 					if ((LA(1)=='u')) {
 						match('u');
 					}
 					else {
-						if ( _cnt635>=1 ) { break _loop635; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+						if ( _cnt663>=1 ) { break _loop663; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 					}
 					
-					_cnt635++;
+					_cnt663++;
 				} while (true);
 				}
 				if ( inputState.guessing==0 ) {
@@ -2060,13 +2068,13 @@ inputState.guessing--;
 			{
 				matchRange('0','3');
 				{
-				if (((LA(1) >= '0' && LA(1) <= '7')) && ((LA(2) >= '\u0003' && LA(2) <= '\ufffe')) && (true) && (true)) {
+				if (((LA(1) >= '0' && LA(1) <= '7')) && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && (true) && (true)) {
 					matchRange('0','7');
 					{
-					if (((LA(1) >= '0' && LA(1) <= '7')) && ((LA(2) >= '\u0003' && LA(2) <= '\ufffe')) && (true) && (true)) {
+					if (((LA(1) >= '0' && LA(1) <= '7')) && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && (true) && (true)) {
 						matchRange('0','7');
 					}
-					else if (((LA(1) >= '\u0003' && LA(1) <= '\ufffe')) && (true) && (true) && (true)) {
+					else if (((LA(1) >= '\u0000' && LA(1) <= '\ufffe')) && (true) && (true) && (true)) {
 					}
 					else {
 						throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
@@ -2074,7 +2082,7 @@ inputState.guessing--;
 					
 					}
 				}
-				else if (((LA(1) >= '\u0003' && LA(1) <= '\ufffe')) && (true) && (true) && (true)) {
+				else if (((LA(1) >= '\u0000' && LA(1) <= '\ufffe')) && (true) && (true) && (true)) {
 				}
 				else {
 					throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
@@ -2090,10 +2098,10 @@ inputState.guessing--;
 			{
 				matchRange('4','7');
 				{
-				if (((LA(1) >= '0' && LA(1) <= '7')) && ((LA(2) >= '\u0003' && LA(2) <= '\ufffe')) && (true) && (true)) {
+				if (((LA(1) >= '0' && LA(1) <= '7')) && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && (true) && (true)) {
 					matchRange('0','7');
 				}
-				else if (((LA(1) >= '\u0003' && LA(1) <= '\ufffe')) && (true) && (true) && (true)) {
+				else if (((LA(1) >= '\u0000' && LA(1) <= '\ufffe')) && (true) && (true) && (true)) {
 				}
 				else {
 					throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
@@ -2162,7 +2170,7 @@ inputState.guessing--;
 		boolean dollarOK = false;
 		
 		{
-		_loop616:
+		_loop644:
 		do {
 			switch ( LA(1)) {
 			case '\\':
@@ -2181,10 +2189,10 @@ inputState.guessing--;
 				break;
 			}
 			default:
-				boolean synPredMatched615 = false;
-				if ((((LA(1)=='"') && ((LA(2) >= '\u0003' && LA(2) <= '\ufffe')) && (true) && (true))&&(tripleQuote))) {
-					int _m615 = mark();
-					synPredMatched615 = true;
+				boolean synPredMatched643 = false;
+				if ((((LA(1)=='"') && ((LA(2) >= '\u0000' && LA(2) <= '\ufffe')) && (true) && (true))&&(tripleQuote))) {
+					int _m643 = mark();
+					synPredMatched643 = true;
 					inputState.guessing++;
 					try {
 						{
@@ -2205,19 +2213,19 @@ inputState.guessing--;
 						}
 					}
 					catch (RecognitionException pe) {
-						synPredMatched615 = false;
+						synPredMatched643 = false;
 					}
-					rewind(_m615);
+					rewind(_m643);
 inputState.guessing--;
 				}
-				if ( synPredMatched615 ) {
+				if ( synPredMatched643 ) {
 					match('"');
 				}
 				else if ((_tokenSet_4.member(LA(1)))) {
 					mSTRING_CH(false);
 				}
 			else {
-				break _loop616;
+				break _loop644;
 			}
 			}
 		} while (true);
@@ -2393,13 +2401,13 @@ inputState.guessing--;
 		
 		}
 		{
-		_loop631:
+		_loop659:
 		do {
 			if ((LA(1)=='*')) {
 				match('*');
 			}
 			else {
-				break _loop631;
+				break _loop659;
 			}
 			
 		} while (true);
@@ -2420,7 +2428,7 @@ inputState.guessing--;
 		int _saveIndex;
 		
 		{
-		_loop625:
+		_loop653:
 		do {
 			if (((LA(1)=='$') && (_tokenSet_2.member(LA(2))))&&(!atValidDollarEscape())) {
 				match('$');
@@ -2429,7 +2437,7 @@ inputState.guessing--;
 				mREGEXP_SYMBOL(false);
 			}
 			else {
-				break _loop625;
+				break _loop653;
 			}
 			
 		} while (true);
@@ -2539,21 +2547,69 @@ inputState.guessing--;
 		_ttype = IDENT;
 		int _saveIndex;
 		
-		mLETTER(false);
 		{
-		_loop645:
-		do {
-			if ((_tokenSet_0.member(LA(1)))) {
+		if (((_tokenSet_0.member(LA(1))) && (true) && (true) && (true))&&(stringCtorState == 0)) {
+			{
+			if ((LA(1)=='$')) {
+				mDOLLAR(false);
+			}
+			else if ((_tokenSet_9.member(LA(1)))) {
 				mLETTER(false);
 			}
-			else if (((LA(1) >= '0' && LA(1) <= '9'))) {
-				mDIGIT(false);
-			}
 			else {
-				break _loop645;
+				throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
 			}
 			
-		} while (true);
+			}
+			{
+			_loop675:
+			do {
+				switch ( LA(1)) {
+				case '0':  case '1':  case '2':  case '3':
+				case '4':  case '5':  case '6':  case '7':
+				case '8':  case '9':
+				{
+					mDIGIT(false);
+					break;
+				}
+				case '$':
+				{
+					mDOLLAR(false);
+					break;
+				}
+				default:
+					if ((_tokenSet_9.member(LA(1)))) {
+						mLETTER(false);
+					}
+				else {
+					break _loop675;
+				}
+				}
+			} while (true);
+			}
+		}
+		else if ((_tokenSet_9.member(LA(1))) && (true) && (true) && (true)) {
+			mLETTER(false);
+			{
+			_loop677:
+			do {
+				if ((_tokenSet_9.member(LA(1)))) {
+					mLETTER(false);
+				}
+				else if (((LA(1) >= '0' && LA(1) <= '9'))) {
+					mDIGIT(false);
+				}
+				else {
+					break _loop677;
+				}
+				
+			} while (true);
+			}
+		}
+		else {
+			throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());
+		}
+		
 		}
 		if ( inputState.guessing==0 ) {
 			
@@ -2574,6 +2630,9 @@ inputState.guessing--;
 			(LA(1) == '.' || lastSigTokenType == DOT || lastSigTokenType == LITERAL_package)) {
 			ttype = IDENT;
 			}
+			if (ttype == LITERAL_static && LA(1) == '.') {
+			ttype = IDENT;
+			}
 			
 			/* The grammar allows a few keywords to follow dot.
 			* TODO: Reinstate this logic if we change or remove keywordPropertyNames.
@@ -2738,40 +2797,40 @@ inputState.guessing--;
 					isDecimal = false;
 				}
 				{
-				int _cnt653=0;
-				_loop653:
+				int _cnt685=0;
+				_loop685:
 				do {
-					if ((_tokenSet_9.member(LA(1))) && (true) && (true) && (true)) {
+					if ((_tokenSet_10.member(LA(1))) && (true) && (true) && (true)) {
 						mHEX_DIGIT(false);
 					}
 					else {
-						if ( _cnt653>=1 ) { break _loop653; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+						if ( _cnt685>=1 ) { break _loop685; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 					}
 					
-					_cnt653++;
+					_cnt685++;
 				} while (true);
 				}
 			}
 			else {
-				boolean synPredMatched659 = false;
+				boolean synPredMatched691 = false;
 				if ((((LA(1) >= '0' && LA(1) <= '9')) && (true) && (true) && (true))) {
-					int _m659 = mark();
-					synPredMatched659 = true;
+					int _m691 = mark();
+					synPredMatched691 = true;
 					inputState.guessing++;
 					try {
 						{
 						{
-						int _cnt656=0;
-						_loop656:
+						int _cnt688=0;
+						_loop688:
 						do {
 							if (((LA(1) >= '0' && LA(1) <= '9'))) {
 								matchRange('0','9');
 							}
 							else {
-								if ( _cnt656>=1 ) { break _loop656; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+								if ( _cnt688>=1 ) { break _loop688; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 							}
 							
-							_cnt656++;
+							_cnt688++;
 						} while (true);
 						}
 						{
@@ -2803,40 +2862,40 @@ inputState.guessing--;
 						}
 					}
 					catch (RecognitionException pe) {
-						synPredMatched659 = false;
+						synPredMatched691 = false;
 					}
-					rewind(_m659);
+					rewind(_m691);
 inputState.guessing--;
 				}
-				if ( synPredMatched659 ) {
+				if ( synPredMatched691 ) {
 					{
-					int _cnt661=0;
-					_loop661:
+					int _cnt693=0;
+					_loop693:
 					do {
 						if (((LA(1) >= '0' && LA(1) <= '9'))) {
 							matchRange('0','9');
 						}
 						else {
-							if ( _cnt661>=1 ) { break _loop661; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+							if ( _cnt693>=1 ) { break _loop693; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 						}
 						
-						_cnt661++;
+						_cnt693++;
 					} while (true);
 					}
 				}
 				else if (((LA(1) >= '0' && LA(1) <= '7')) && (true) && (true) && (true)) {
 					{
-					int _cnt663=0;
-					_loop663:
+					int _cnt695=0;
+					_loop695:
 					do {
 						if (((LA(1) >= '0' && LA(1) <= '7'))) {
 							matchRange('0','7');
 						}
 						else {
-							if ( _cnt663>=1 ) { break _loop663; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+							if ( _cnt695>=1 ) { break _loop695; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 						}
 						
-						_cnt663++;
+						_cnt695++;
 					} while (true);
 					}
 					if ( inputState.guessing==0 ) {
@@ -2857,13 +2916,13 @@ inputState.guessing--;
 				matchRange('1','9');
 				}
 				{
-				_loop666:
+				_loop698:
 				do {
 					if (((LA(1) >= '0' && LA(1) <= '9'))) {
 						matchRange('0','9');
 					}
 					else {
-						break _loop666;
+						break _loop698;
 					}
 					
 				} while (true);
@@ -2940,14 +2999,14 @@ inputState.guessing--;
 				break;
 			}
 			default:
-				boolean synPredMatched672 = false;
+				boolean synPredMatched704 = false;
 				if ((((LA(1)=='.'||LA(1)=='D'||LA(1)=='E'||LA(1)=='F'||LA(1)=='d'||LA(1)=='e'||LA(1)=='f'))&&(isDecimal))) {
-					int _m672 = mark();
-					synPredMatched672 = true;
+					int _m704 = mark();
+					synPredMatched704 = true;
 					inputState.guessing++;
 					try {
 						{
-						if ((_tokenSet_10.member(LA(1)))) {
+						if ((_tokenSet_11.member(LA(1)))) {
 							matchNot('.');
 						}
 						else if ((LA(1)=='.')) {
@@ -2963,29 +3022,29 @@ inputState.guessing--;
 						}
 					}
 					catch (RecognitionException pe) {
-						synPredMatched672 = false;
+						synPredMatched704 = false;
 					}
-					rewind(_m672);
+					rewind(_m704);
 inputState.guessing--;
 				}
-				if ( synPredMatched672 ) {
+				if ( synPredMatched704 ) {
 					{
 					switch ( LA(1)) {
 					case '.':
 					{
 						match('.');
 						{
-						int _cnt675=0;
-						_loop675:
+						int _cnt707=0;
+						_loop707:
 						do {
 							if (((LA(1) >= '0' && LA(1) <= '9'))) {
 								matchRange('0','9');
 							}
 							else {
-								if ( _cnt675>=1 ) { break _loop675; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+								if ( _cnt707>=1 ) { break _loop707; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 							}
 							
-							_cnt675++;
+							_cnt707++;
 						} while (true);
 						}
 						{
@@ -3140,17 +3199,17 @@ inputState.guessing--;
 		}
 		}
 		{
-		int _cnt684=0;
-		_loop684:
+		int _cnt716=0;
+		_loop716:
 		do {
 			if (((LA(1) >= '0' && LA(1) <= '9'))) {
 				matchRange('0','9');
 			}
 			else {
-				if ( _cnt684>=1 ) { break _loop684; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
+				if ( _cnt716>=1 ) { break _loop716; } else {throw new NoViableAltForCharException((char)LA(1), getFilename(), getLine(), getColumn());}
 			}
 			
-			_cnt684++;
+			_cnt716++;
 		} while (true);
 		}
 		if ( _createToken && _token==null && _ttype!=Token.SKIP ) {
@@ -3242,6 +3301,7 @@ inputState.guessing--;
 	
 	private static final long[] mk_tokenSet_0() {
 		long[] data = new long[2560];
+		data[0]=68719476736L;
 		data[1]=576460745995190270L;
 		data[3]=-36028797027352577L;
 		for (int i = 4; i<=1022; i++) { data[i]=-1L; }
@@ -3251,7 +3311,7 @@ inputState.guessing--;
 	public static final BitSet _tokenSet_0 = new BitSet(mk_tokenSet_0());
 	private static final long[] mk_tokenSet_1() {
 		long[] data = new long[2048];
-		data[0]=-9224L;
+		data[0]=-9217L;
 		for (int i = 1; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
 		return data;
@@ -3259,7 +3319,7 @@ inputState.guessing--;
 	public static final BitSet _tokenSet_1 = new BitSet(mk_tokenSet_1());
 	private static final long[] mk_tokenSet_2() {
 		long[] data = new long[2048];
-		data[0]=-4398046520328L;
+		data[0]=-4398046520321L;
 		for (int i = 1; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
 		return data;
@@ -3267,14 +3327,14 @@ inputState.guessing--;
 	public static final BitSet _tokenSet_2 = new BitSet(mk_tokenSet_2());
 	private static final long[] mk_tokenSet_3() {
 		long[] data = new long[2048];
-		data[0]=-549755813896L;
+		data[0]=-549755813889L;
 		for (int i = 1; i<=1023; i++) { data[i]=-1L; }
 		return data;
 	}
 	public static final BitSet _tokenSet_3 = new BitSet(mk_tokenSet_3());
 	private static final long[] mk_tokenSet_4() {
 		long[] data = new long[2048];
-		data[0]=-635655169032L;
+		data[0]=-635655169025L;
 		data[1]=-268435457L;
 		for (int i = 2; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
@@ -3283,14 +3343,14 @@ inputState.guessing--;
 	public static final BitSet _tokenSet_4 = new BitSet(mk_tokenSet_4());
 	private static final long[] mk_tokenSet_5() {
 		long[] data = new long[2048];
-		data[0]=-17179869192L;
+		data[0]=-17179869185L;
 		for (int i = 1; i<=1023; i++) { data[i]=-1L; }
 		return data;
 	}
 	public static final BitSet _tokenSet_5 = new BitSet(mk_tokenSet_5());
 	private static final long[] mk_tokenSet_6() {
 		long[] data = new long[2048];
-		data[0]=-145135534875656L;
+		data[0]=-145135534875649L;
 		for (int i = 1; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
 		return data;
@@ -3298,7 +3358,7 @@ inputState.guessing--;
 	public static final BitSet _tokenSet_6 = new BitSet(mk_tokenSet_6());
 	private static final long[] mk_tokenSet_7() {
 		long[] data = new long[2048];
-		data[0]=-145204254352392L;
+		data[0]=-145204254352385L;
 		for (int i = 1; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
 		return data;
@@ -3306,7 +3366,7 @@ inputState.guessing--;
 	public static final BitSet _tokenSet_7 = new BitSet(mk_tokenSet_7());
 	private static final long[] mk_tokenSet_8() {
 		long[] data = new long[2048];
-		data[0]=-145204254352392L;
+		data[0]=-145204254352385L;
 		data[1]=-268435457L;
 		for (int i = 2; i<=1022; i++) { data[i]=-1L; }
 		data[1023]=9223372036854775807L;
@@ -3314,18 +3374,27 @@ inputState.guessing--;
 	}
 	public static final BitSet _tokenSet_8 = new BitSet(mk_tokenSet_8());
 	private static final long[] mk_tokenSet_9() {
+		long[] data = new long[2560];
+		data[1]=576460745995190270L;
+		data[3]=-36028797027352577L;
+		for (int i = 4; i<=1022; i++) { data[i]=-1L; }
+		data[1023]=9223372036854775807L;
+		return data;
+	}
+	public static final BitSet _tokenSet_9 = new BitSet(mk_tokenSet_9());
+	private static final long[] mk_tokenSet_10() {
 		long[] data = new long[1025];
 		data[0]=287948901175001088L;
 		data[1]=541165879422L;
 		return data;
 	}
-	public static final BitSet _tokenSet_9 = new BitSet(mk_tokenSet_9());
-	private static final long[] mk_tokenSet_10() {
+	public static final BitSet _tokenSet_10 = new BitSet(mk_tokenSet_10());
+	private static final long[] mk_tokenSet_11() {
 		long[] data = new long[2048];
-		data[0]=-70368744177672L;
+		data[0]=-70368744177665L;
 		for (int i = 1; i<=1023; i++) { data[i]=-1L; }
 		return data;
 	}
-	public static final BitSet _tokenSet_10 = new BitSet(mk_tokenSet_10());
+	public static final BitSet _tokenSet_11 = new BitSet(mk_tokenSet_11());
 	
 	}
