@@ -1,4 +1,4 @@
-// $ANTLR 2.7.6 (2005-12-22): "groovy.g" -> "GroovyRecognizer.java"$
+// $ANTLR 2.7.7 (20060906): "groovy.g" -> "GroovyRecognizer.java"$
 
 package org.codehaus.groovy.antlr.parser;
 import org.codehaus.groovy.antlr.*;
@@ -8,6 +8,8 @@ import java.io.Reader;
 import antlr.InputBuffer;
 import antlr.LexerSharedInputState;
 import antlr.CommonToken;
+import org.codehaus.groovy.GroovyBugError;
+import antlr.TokenStreamRecognitionException;
 
 public interface GroovyTokenTypes {
 	int EOF = 1;
@@ -161,47 +163,47 @@ public interface GroovyTokenTypes {
 	int SPREAD_DOT = 150;
 	int OPTIONAL_DOT = 151;
 	int MEMBER_POINTER = 152;
-	int PLUS_ASSIGN = 153;
-	int MINUS_ASSIGN = 154;
-	int STAR_ASSIGN = 155;
-	int DIV_ASSIGN = 156;
-	int MOD_ASSIGN = 157;
-	int SR_ASSIGN = 158;
-	int BSR_ASSIGN = 159;
-	int SL_ASSIGN = 160;
-	int BAND_ASSIGN = 161;
-	int BXOR_ASSIGN = 162;
-	int BOR_ASSIGN = 163;
-	int STAR_STAR_ASSIGN = 164;
-	int ELVIS_OPERATOR = 165;
-	int LOR = 166;
-	int LAND = 167;
-	int BOR = 168;
-	int BXOR = 169;
-	int REGEX_FIND = 170;
-	int REGEX_MATCH = 171;
-	int NOT_EQUAL = 172;
-	int EQUAL = 173;
-	int COMPARE_TO = 174;
-	int LE = 175;
-	int GE = 176;
-	int LITERAL_instanceof = 177;
-	int SL = 178;
-	int RANGE_INCLUSIVE = 179;
-	int RANGE_EXCLUSIVE = 180;
-	int INC = 181;
-	int DIV = 182;
-	int MOD = 183;
-	int DEC = 184;
-	int STAR_STAR = 185;
-	int BNOT = 186;
-	int LNOT = 187;
-	int STRING_CTOR_START = 188;
-	int STRING_CTOR_END = 189;
-	int LITERAL_new = 190;
-	int LITERAL_true = 191;
-	int LITERAL_false = 192;
-	int LITERAL_null = 193;
+	int LITERAL_false = 153;
+	int LITERAL_instanceof = 154;
+	int LITERAL_new = 155;
+	int LITERAL_null = 156;
+	int LITERAL_true = 157;
+	int PLUS_ASSIGN = 158;
+	int MINUS_ASSIGN = 159;
+	int STAR_ASSIGN = 160;
+	int DIV_ASSIGN = 161;
+	int MOD_ASSIGN = 162;
+	int SR_ASSIGN = 163;
+	int BSR_ASSIGN = 164;
+	int SL_ASSIGN = 165;
+	int BAND_ASSIGN = 166;
+	int BXOR_ASSIGN = 167;
+	int BOR_ASSIGN = 168;
+	int STAR_STAR_ASSIGN = 169;
+	int ELVIS_OPERATOR = 170;
+	int LOR = 171;
+	int LAND = 172;
+	int BOR = 173;
+	int BXOR = 174;
+	int REGEX_FIND = 175;
+	int REGEX_MATCH = 176;
+	int NOT_EQUAL = 177;
+	int EQUAL = 178;
+	int COMPARE_TO = 179;
+	int LE = 180;
+	int GE = 181;
+	int SL = 182;
+	int RANGE_INCLUSIVE = 183;
+	int RANGE_EXCLUSIVE = 184;
+	int INC = 185;
+	int DIV = 186;
+	int MOD = 187;
+	int DEC = 188;
+	int STAR_STAR = 189;
+	int BNOT = 190;
+	int LNOT = 191;
+	int STRING_CTOR_START = 192;
+	int STRING_CTOR_END = 193;
 	int NUM_INT = 194;
 	int NUM_FLOAT = 195;
 	int NUM_LONG = 196;
