@@ -28,15 +28,15 @@ import antlr.collections.AST;
  * Helper Class for Antlr AST traversal and visitation.
  *
  * @author <a href="mailto:groovy@ross-rayner.com">Jeremy Rayner</a>
- * @version $Revision: 8794 $
+ * @version $Revision: 17581 $
  */
 
 public abstract class TraversalHelper implements AntlrASTProcessor {
-    protected List unvisitedNodes;
+    protected List<GroovySourceAST> unvisitedNodes;
     private final Visitor v;
 
     public TraversalHelper(Visitor visitor) {
-        this.unvisitedNodes = new ArrayList();
+        this.unvisitedNodes = new ArrayList<GroovySourceAST>();
         this.v = visitor;
     }
 
@@ -316,12 +316,20 @@ public abstract class TraversalHelper implements AntlrASTProcessor {
         subsequentVisit(t);
         accept(t.childAt(1));
     }
+    
     protected void accept_FirstChild_v_SecondChild_v(GroovySourceAST t) {
         accept(t.childAt(0));
         openingVisit(t);
         accept(t.childAt(1));
         closingVisit(t);
     }
+    
+    protected void accept_SecondChild_v_ThirdChild_v(GroovySourceAST t) {
+        accept(t.childAt(1));
+        openingVisit(t);
+        accept(t.childAt(2));
+        closingVisit(t);
+    }
 
     protected void accept_FirstChild_v_SecondChildsChildren_v(GroovySourceAST t) {
         accept(t.childAt(0));
