@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2009 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,10 +16,12 @@
 package org.codehaus.groovy.ast;
 
 import org.codehaus.groovy.GroovyBugError;
-import org.codehaus.groovy.ast.expr.ClassExpression;
+import org.codehaus.groovy.ast.expr.BinaryExpression;
 import org.codehaus.groovy.ast.expr.Expression;
+import org.codehaus.groovy.ast.expr.FieldExpression;
 import org.codehaus.groovy.ast.expr.MapExpression;
 import org.codehaus.groovy.ast.expr.TupleExpression;
+import org.codehaus.groovy.ast.stmt.ExpressionStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
 import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.control.CompilePhase;
@@ -49,37 +51,37 @@ import groovy.lang.GroovyObject;
  * <ol>
  * <li> Primary ClassNodes:<br>
  * A primary ClassNode is one where we have a source representation
- * which is to be compiled by Groovy and which we have an AST for.
+ * which is to be compiled by Groovy and which we have an AST for. 
  * The groovy compiler will output one class for each such ClassNode
  * that passes through AsmBytecodeGenerator... not more, not less.
  * That means for example Closures become such ClassNodes too at
- * some point.
- *
+ * some point. 
+ * 
  * <li> ClassNodes create through different sources (typically created
  * from a java.lang.reflect.Class object):<br>
  * The compiler will not output classes from these, the methods
  * usually do not contain bodies. These kind of ClassNodes will be
- * used in different checks, but not checks that work on the method
+ * used in different checks, but not checks that work on the method 
  * bodies. For example if such a ClassNode is a super class to a primary
- * ClassNode, then the abstract method test and others will be done
- * with data based on these. Theoretically it is also possible to mix both
+ * ClassNode, then the abstract method test and others will be done 
+ * with data based on these. Theoretically it is also possible to mix both 
  * (1 and 2) kind of classes in a hierarchy, but this probably works only
  *  in the newest Groovy versions. Such ClassNodes normally have to
- *  isResolved() returning true without having a redirect.In the Groovy
- *  compiler the only version of this, that exists, is a ClassNode created
+ *  isResolved() returning true without having a redirect.In the Groovy 
+ *  compiler the only version of this, that exists, is a ClassNode created 
  *  through a Class instance
  *
  * <li> Labels:<br>
- * ClassNodes created through ClassHelper.makeWithoutCaching. They
+ * ClassNodes created through ClassHelper.makeWithoutCaching. They 
  * are place holders, its redirect points to the real structure, which can
  * be a label too, but following all redirects it should end with a ClassNode
- * from one of the other two categories. If ResolveVisitor finds such a
- * node, it tries to set the redirects. Any such label created after
- * ResolveVisitor has done its work needs to have a redirect pointing to
- * case 1 or 2. If not the compiler may react strange... this can be considered
- * as a kind of dangling pointer.
+ * from one of the other two categories. If ResolveVisitor finds such a 
+ * node, it tries to set the redirects. Any such label created after 
+ * ResolveVisitor has done its work needs to have a redirect pointing to 
+ * case 1 or 2. If not the compiler may react strange... this can be considered 
+ * as a kind of dangling pointer. 
  * <br>
- * <b>Note:</b> the redirect mechanism is only allowed for classes
+ * <b>Note:</b> the redirect mechanism is only allowed for classes 
  * that are not primary ClassNodes. Typically this is done for classes
  * created by name only.  The redirect itself can be any type of ClassNode.
  * <br>
@@ -93,46 +95,45 @@ import groovy.lang.GroovyObject;
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
  * @author Jochen Theodorou
- * @version $Revision: 16733 $
+ * @version $Revision: 18396 $
  */
 public class ClassNode extends AnnotatedNode implements Opcodes {
     private static class MapOfLists {
-        private Map map = new HashMap();
-        public List get(Object key) {
-            return (List) map.get(key);
+        private Map<Object, List<MethodNode>> map = new HashMap<Object, List<MethodNode>>();
+        public List<MethodNode> get(Object key) {
+            return map.get(key);
         }
-        public List getNotNull(Object key) {
-            List ret = get(key);
-            if (ret==null) ret = Collections.EMPTY_LIST;
+        public List<MethodNode> getNotNull(Object key) {
+            List<MethodNode> ret = get(key);
+            if (ret==null) ret = Collections.emptyList();
             return ret;
         }
-        public void put(Object key, Object value) {
+        public void put(Object key, MethodNode value) {
             if (map.containsKey(key)) {
                 get(key).add(value);
             } else {
-                ArrayList list = new ArrayList(2);
+                ArrayList<MethodNode> list = new ArrayList<MethodNode>(2);
                 list.add(value);
                 map.put(key, list);
             }
         }
     }
 
-    public static ClassNode[] EMPTY_ARRAY = new ClassNode[0];
-
-    public static ClassNode THIS = new ClassNode(Object.class);
-    public static ClassNode SUPER = new ClassNode(Object.class);
+    public static final ClassNode[] EMPTY_ARRAY = new ClassNode[0];
+    public static final ClassNode THIS = new ClassNode(Object.class);
+    public static final ClassNode SUPER = new ClassNode(Object.class);
 
     private String name;
     private int modifiers;
     private ClassNode[] interfaces;
     private MixinNode[] mixins;
-    private List constructors;
-    private List  objectInitializers;
+    private List<ConstructorNode> constructors;
+    private List<Statement> objectInitializers;
     private MapOfLists methods;
     private List<MethodNode> methodsList;
     private LinkedList<FieldNode> fields;
-    private List properties;
-    private Map fieldIndex;
+    private List<PropertyNode> properties;
+    private Map<String, FieldNode> fieldIndex;
     private ModuleNode module;
     private CompileUnit compileUnit;
     private boolean staticClass = false;
@@ -140,13 +141,13 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     private boolean script;
     private ClassNode superClass;
     protected boolean isPrimaryNode;
+    protected List<InnerClassNode> innerClasses;
 
     /**
      * The ASTTransformations to be applied to the Class
      */
     private Map<CompilePhase, Map<Class<? extends ASTTransformation>, Set<ASTNode>>> transformInstances;
 
-
     // use this to synchronize access for the lazy init
     protected Object lazyInitLock = new Object();
 
@@ -174,9 +175,8 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * Returns the ClassNode this ClassNode is redirecting to.
      */
     public ClassNode redirect(){
-        ClassNode res = this;
-        while (res.redirect != null) res = res.redirect;
-        return res;
+        if (redirect==null) return this;
+        return redirect.redirect();
     }
 
     /**
@@ -195,7 +195,11 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * represented by this ClassNode
      */
     public ClassNode makeArray() {
-        if (redirect!=null) return redirect().makeArray();
+        if (redirect!=null) {
+            ClassNode res = redirect().makeArray();
+            res.componentType = this;
+            return res;
+        }
         ClassNode cn;
         if (clazz!=null) {
             Class ret = Array.newInstance(clazz,0).getClass();
@@ -208,13 +212,13 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     /**
-     * Returns if this instance is a primary ClassNode
+     * @return true if this instance is a primary ClassNode
      */
     public boolean isPrimaryClassNode(){
-    return redirect().isPrimaryNode || (componentType!= null && componentType.isPrimaryClassNode());
+    	return redirect().isPrimaryNode || (componentType!= null && componentType.isPrimaryClassNode());
     }
 
-    /**
+    /*
      * Constructor used by makeArray() if no real class is available
      */
     private ClassNode(ClassNode componentType) {
@@ -223,7 +227,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         isPrimaryNode=false;
     }
 
-    /**
+    /*
      * Constructor used by makeArray() if a real class is available
      */
     private ClassNode(Class c, ClassNode componentType) {
@@ -254,7 +258,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
             if (redirect!=null) {
                 throw new GroovyBugError("lazyClassInit called on a proxy ClassNode, that must not happen."+
                                          "A redirect() call is missing somewhere!");
-            }
+            }   
             if (lazyInitDone) return;
             VMPluginFactory.getPlugin().configureClassNode(compileUnit,this);
             lazyInitDone = true;
@@ -289,6 +293,8 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @param modifiers  the modifiers,
      * @param superClass the base class name - use "java.lang.Object" if no direct
      *                   base class
+     * @param interfaces the interfaces for this class
+     * @param mixins     the mixins for this class
      * @see org.objectweb.asm.Opcodes
      */
     public ClassNode(String name, int modifiers, ClassNode superClass, ClassNode[] interfaces, MixinNode[] mixins) {
@@ -302,16 +308,13 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
             usesGenerics = superClass.isUsingGenerics();
         }
         if (!usesGenerics && interfaces!=null) {
-            for (int i = 0; i < interfaces.length; i++) {
-                usesGenerics = usesGenerics || interfaces[i].isUsingGenerics();
+            for (ClassNode anInterface : interfaces) {
+                usesGenerics = usesGenerics || anInterface.isUsingGenerics();
             }
         }
+        this.methods = new MapOfLists();
+        this.methodsList = new ArrayList<MethodNode>();
 
-        if ((modifiers & ACC_INTERFACE) == 0)
-          addField("$ownClass", ACC_STATIC|ACC_PUBLIC|ACC_FINAL|ACC_SYNTHETIC, ClassHelper.CLASS_Type, new ClassExpression(this)).setSynthetic(true);
-    }
-
-    private void getTransformInstancesLazy() {
         transformInstances = new EnumMap<CompilePhase, Map<Class <? extends ASTTransformation>, Set<ASTNode>>>(CompilePhase.class);
         for (CompilePhase phase : CompilePhase.values()) {
             transformInstances.put(phase, new HashMap<Class <? extends ASTTransformation>, Set<ASTNode>>());
@@ -326,25 +329,18 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     /**
-     * Returns a list containing FieldNode objects for
-     * each field in the class represented by this ClassNode
+     * @return the list of FieldNode's associated with this ClassNode
      */
     public List<FieldNode> getFields() {
         if (!redirect().lazyInitDone) redirect().lazyClassInit();
         if (redirect!=null) return redirect().getFields();
-        return getFieldsLazy();
-    }
-
-    private List<FieldNode> getFieldsLazy() {
-        if (fields == null) {
-            fields = new LinkedList<FieldNode>();
-        }
+        if (fields == null)
+            fields = new LinkedList<FieldNode> ();
         return fields;
     }
 
     /**
-     * Returns an array of ClassNodes representing the
-     * interfaces the class implements
+     * @return the array of interfaces which this ClassNode implements
      */
     public ClassNode[] getInterfaces() {
         if (!redirect().lazyInitDone) redirect().lazyClassInit();
@@ -360,35 +356,34 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         }
     }
 
+    /**
+     * @return the array of mixins associated with this ClassNode
+     */
     public MixinNode[] getMixins() {
         return redirect().mixins;
     }
 
     /**
-     * Returns a list containing MethodNode objects for
-     * each method in the class represented by this ClassNode
+     * @return the list of methods associated with this ClassNode
      */
     public List<MethodNode> getMethods() {
         if (!redirect().lazyInitDone) redirect().lazyClassInit();
         if (redirect!=null) return redirect().getMethods();
-        return getMethodsListLazy();
+        return methodsList;
     }
 
     /**
-     * Returns a list containing MethodNode objects for
-     * each abstract method in the class represented by
-     * this ClassNode
+     * @return the list of abstract methods associated with this
+     * ClassNode or null if there are no such methods
      */
-    public List getAbstractMethods() {
-        List result = new ArrayList(3);
-        Map declaredMethods = getDeclaredMethodsMap();
-        for (Iterator it = declaredMethods.values().iterator(); it.hasNext();) {
-            MethodNode method = (MethodNode) it.next();
+    public List<MethodNode> getAbstractMethods() {
+        List<MethodNode> result = new ArrayList<MethodNode>(3);
+        for (MethodNode method : getDeclaredMethodsMap().values()) {
             if (method.isAbstract()) {
                 result.add(method);
             }
         }
-
+        
         if (result.isEmpty()) {
             return null;
         } else {
@@ -396,54 +391,49 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         }
     }
 
-    public List getAllDeclaredMethods() {
-        return new ArrayList(getDeclaredMethodsMap().values());
+    public List<MethodNode> getAllDeclaredMethods() {
+        return new ArrayList<MethodNode>(getDeclaredMethodsMap().values());
     }
 
-    public Set getAllInterfaces () {
-        Set res = new HashSet ();
+    public Set<ClassNode> getAllInterfaces () {
+        Set<ClassNode> res = new HashSet<ClassNode>();
         getAllInterfaces(res);
         return res;
     }
 
-    private void getAllInterfaces(Set res) {
+    private void getAllInterfaces(Set<ClassNode> res) {
         if (isInterface())
           res.add(this);
 
-        ClassNode[] interfaces = getInterfaces();
-        for (int i = 0; i < interfaces.length; i++) {
-            res.add(interfaces[i]);
-            interfaces[i].getAllInterfaces(res);
+        for (ClassNode anInterface : getInterfaces()) {
+            res.add(anInterface);
+            anInterface.getAllInterfaces(res);
         }
     }
 
-    public Map getDeclaredMethodsMap() {
+    public Map<String, MethodNode> getDeclaredMethodsMap() {
         // Start off with the methods from the superclass.
         ClassNode parent = getSuperClass();
-        Map result = null;
+        Map<String, MethodNode> result = null;
         if (parent != null) {
             result = parent.getDeclaredMethodsMap();
         } else {
-            result = new HashMap();
+            result = new HashMap<String, MethodNode>();
         }
 
         // add in unimplemented abstract methods from the interfaces
-        ClassNode[] interfaces = getInterfaces();
-        for (int i = 0; i < interfaces.length; i++) {
-            ClassNode iface = interfaces[i];
-            Map ifaceMethodsMap = iface.getDeclaredMethodsMap();
-            for (Object o : ifaceMethodsMap.keySet()) {
-                String methSig = (String) o;
+        for (ClassNode iface : getInterfaces()) {
+            Map<String, MethodNode> ifaceMethodsMap = iface.getDeclaredMethodsMap();
+            for (String methSig : ifaceMethodsMap.keySet()) {
                 if (!result.containsKey(methSig)) {
-                    MethodNode methNode = (MethodNode) ifaceMethodsMap.get(methSig);
+                    MethodNode methNode = ifaceMethodsMap.get(methSig);
                     result.put(methSig, methNode);
                 }
             }
         }
 
         // And add in the methods implemented in this class.
-        for (Object o : getMethods()) {
-            MethodNode method = (MethodNode) o;
+        for (MethodNode method : getMethods()) {
             String sig = method.getTypeDescriptor();
             result.put(sig, method);
         }
@@ -454,6 +444,10 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         return redirect().name;
     }
 
+    public String getUnresolvedName() {
+        return name;
+    }
+
     public String setName(String name) {
         return redirect().name=name;
     }
@@ -466,31 +460,29 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         redirect().modifiers = modifiers;
     }
 
-    public List getProperties() {
-        return redirect().getPropertiesLazy();
+    public List<PropertyNode> getProperties() {
+        final ClassNode r = redirect();
+        if (r.properties == null)
+            r.properties = new ArrayList<PropertyNode> ();
+        return r.properties;
     }
 
-    private List getPropertiesLazy() {
-        if (properties == null)
-            properties = new LinkedList ();
-        return properties;
-    }
-
-    public List getDeclaredConstructors() {
+    public List<ConstructorNode> getDeclaredConstructors() {
         if (!redirect().lazyInitDone) redirect().lazyClassInit();
-        return redirect().getDeclaredConstructorsLazy();
-    }
-
-    private List getDeclaredConstructorsLazy () {
-        if (constructors == null)
-            constructors = new LinkedList();
-        return constructors;
+        final ClassNode r = redirect();
+        if (r.constructors == null)
+            r.constructors = new ArrayList<ConstructorNode> ();
+        return r.constructors;
     }
 
     public ModuleNode getModule() {
         return redirect().module;
     }
 
+    public PackageNode getPackage() {
+        return getModule() == null ? null : getModule().getPackage();
+    }
+
     public void setModule(ModuleNode module) {
         redirect().module = module;
         if (module != null) {
@@ -499,23 +491,37 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     public void addField(FieldNode node) {
-        node.setDeclaringClass(redirect());
-        node.setOwner(redirect());
-        redirect().getFieldsLazy().add(node);
-        redirect().getFieldIndexLazy().put(node.getName(), node);
+        final ClassNode r = redirect();
+        node.setDeclaringClass(r);
+        node.setOwner(r);
+        if (r.fields == null)
+            r.fields = new LinkedList<FieldNode> ();
+        if (r.fieldIndex == null)
+            r.fieldIndex = new HashMap<String,FieldNode> ();
+        r.fields.add(node);
+        r.fieldIndex.put(node.getName(), node);
     }
 
-    private Map getFieldIndexLazy() {
-        if (fieldIndex == null)
-            fieldIndex = new HashMap();
-        return fieldIndex;
+    public void addFieldFirst(FieldNode node) {
+        final ClassNode r = redirect();
+        node.setDeclaringClass(r);
+        node.setOwner(r);
+        if (r.fields == null)
+            r.fields = new LinkedList<FieldNode> ();
+        if (r.fieldIndex == null)
+            r.fieldIndex = new HashMap<String,FieldNode> ();
+        r.fields.addFirst(node);
+        r.fieldIndex.put(node.getName(), node);
     }
 
     public void addProperty(PropertyNode node) {
         node.setDeclaringClass(redirect());
         FieldNode field = node.getField();
         addField(field);
-        redirect().getPropertiesLazy().add(node);
+        final ClassNode r = redirect();
+        if (r.properties == null)
+            r.properties = new ArrayList<PropertyNode> ();
+        r.properties.add(node);
     }
 
     public PropertyNode addProperty(String name,
@@ -524,8 +530,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
                                     Expression initialValueExpression,
                                     Statement getterBlock,
                                     Statement setterBlock) {
-        for (Object o : getProperties()) {
-            PropertyNode pn = (PropertyNode) o;
+        for (PropertyNode pn : getProperties()) {
             if (pn.getName().equals(name)) {
                 if (pn.getInitialExpression() == null && initialValueExpression != null)
                     pn.getField().setInitialValueExpression(initialValueExpression);
@@ -548,18 +553,20 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     public boolean hasProperty(String name) {
     	return getProperty(name)!=null;
     }
-
+    
     public PropertyNode getProperty(String name) {
-    	for (Object o : getProperties()) {
-            PropertyNode pn = (PropertyNode) o;
+    	for (PropertyNode pn : getProperties()) {
             if (pn.getName().equals(name)) return pn;
         }
-        return null;
+        return null;   	
     }
 
     public void addConstructor(ConstructorNode node) {
         node.setDeclaringClass(this);
-        redirect().getDeclaredConstructorsLazy().add(node);
+        final ClassNode r = redirect();
+        if (r.constructors == null)
+            r.constructors = new ArrayList<ConstructorNode> ();
+        r.constructors.add(node);
     }
 
     public ConstructorNode addConstructor(int modifiers, Parameter[] parameters, ClassNode[] exceptions, Statement code) {
@@ -570,20 +577,8 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
 
     public void addMethod(MethodNode node) {
         node.setDeclaringClass(this);
-        redirect().getMethodsListLazy().add(node);
-        redirect().getMethodsLazy().put(node.getName(), node);
-    }
-
-    private MapOfLists getMethodsLazy() {
-        if (methods == null)
-            methods = new MapOfLists();
-        return methods;
-    }
-
-    private List<MethodNode> getMethodsListLazy() {
-        if (methodsList == null)
-            methodsList = new LinkedList<MethodNode>();
-        return methodsList;
+        redirect().methodsList.add(node);
+        redirect().methods.put(node.getName(), node);
     }
 
     /**
@@ -645,6 +640,12 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         return node;
     }
 
+    public FieldNode addFieldFirst(String name, int modifiers, ClassNode type, Expression initialValue) {
+        FieldNode node = new FieldNode(name, modifiers, type, redirect(), initialValue);
+        addFieldFirst(node);
+        return node;
+    }
+
     public void addInterface(ClassNode type) {
         // lets check if it already implements an interface
         boolean skip = false;
@@ -697,7 +698,10 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @return the method matching the given name and parameters or null
      */
     public FieldNode getDeclaredField(String name) {
-        return (FieldNode) redirect().getFieldIndexLazy().get(name);
+        ClassNode r = redirect ();
+        if (r.fieldIndex == null)
+            r.fieldIndex = new HashMap<String,FieldNode> ();
+        return r.fieldIndex.get(name);
     }
 
     /**
@@ -731,19 +735,22 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         return null;
     }
 
+    /**
+     * Adds a statement to the object initializer.
+     *
+     * @param statements the statement to be added
+     */
     public void addObjectInitializerStatements(Statement statements) {
-        if (objectInitializers == null)
-            objectInitializers = new LinkedList();
-        objectInitializers.add(statements);
+        getObjectInitializerStatements().add(statements);
     }
 
-    public List getObjectInitializerStatements() {
+    public List<Statement> getObjectInitializerStatements() {
         if (objectInitializers == null)
-            objectInitializers = new LinkedList();
+            objectInitializers = new ArrayList<Statement> ();
         return objectInitializers;
     }
 
-    public void addStaticInitializerStatements(List staticStatements, boolean fieldInit) {
+    private MethodNode getOrAddStaticConstructorNode() {
         MethodNode method = null;
         List declaredMethods = getDeclaredMethods("<clinit>");
         if (declaredMethods.isEmpty()) {
@@ -754,6 +761,11 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         else {
             method = (MethodNode) declaredMethods.get(0);
         }
+        return method;
+    }
+    
+    public void addStaticInitializerStatements(List<Statement> staticStatements, boolean fieldInit) {
+        MethodNode method = getOrAddStaticConstructorNode();
         BlockStatement block = null;
         Statement statement = method.getCode();
         if (statement == null) {
@@ -774,23 +786,50 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         if (!fieldInit) {
             block.addStatements(staticStatements);
         } else {
-            List blockStatements = block.getStatements();
+            List<Statement> blockStatements = block.getStatements();
             staticStatements.addAll(blockStatements);
             blockStatements.clear();
             blockStatements.addAll(staticStatements);
         }
     }
 
+    public void positionStmtsAfterEnumInitStmts(List<Statement> staticFieldStatements) {
+        MethodNode method = getOrAddStaticConstructorNode();
+        Statement statement = method.getCode();
+        if (statement instanceof BlockStatement) {
+        	BlockStatement block = (BlockStatement) statement;
+        	// add given statements for explicitly declared static fields just after enum-special fields
+        	// are found - the $VALUES binary expression marks the end of such fields.
+        	List<Statement> blockStatements = block.getStatements();
+        	ListIterator<Statement> litr = blockStatements.listIterator();
+        	while(litr.hasNext()) {
+        		Statement stmt = litr.next();
+        		if(stmt instanceof ExpressionStatement && 
+        				((ExpressionStatement)stmt).getExpression() instanceof BinaryExpression) {
+        			BinaryExpression bExp = (BinaryExpression) ((ExpressionStatement)stmt).getExpression();
+        			if (bExp.getLeftExpression() instanceof FieldExpression) {
+        				FieldExpression fExp = (FieldExpression) bExp.getLeftExpression();
+        				if(fExp.getFieldName().equals("$VALUES")) {
+        					for(Statement tmpStmt : staticFieldStatements) {
+        						litr.add(tmpStmt);
+        					}
+        				}
+        			}
+        		}
+        	}
+        }
+    }
+    
     /**
      * This methods returns a list of all methods of the given name
      * defined in the current class
      * @return the method list
      * @see #getMethods(String)
      */
-    public List getDeclaredMethods(String name) {
+    public List<MethodNode> getDeclaredMethods(String name) {
         if (!redirect().lazyInitDone) redirect().lazyClassInit();
         if (redirect!=null) return redirect().getDeclaredMethods(name);
-        return getMethodsLazy().getNotNull(name);
+        return methods.getNotNull(name);
     }
 
     /**
@@ -799,8 +838,8 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @return the methods list
      * @see #getDeclaredMethods(String)
      */
-    public List getMethods(String name) {
-        List answer = new ArrayList();
+    public List<MethodNode> getMethods(String name) {
+        List<MethodNode> answer = new ArrayList<MethodNode>();
         ClassNode node = this;
         while (node != null) {
             answer.addAll(node.getDeclaredMethods(name));
@@ -815,8 +854,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @return the method matching the given name and parameters or null
      */
     public MethodNode getDeclaredMethod(String name, Parameter[] parameters) {
-        for (Object o :  getDeclaredMethods(name)) {
-            MethodNode method = (MethodNode) o;
+        for (MethodNode method :  getDeclaredMethods(name)) {
             if (parametersEqual(method.getParameters(), parameters)) {
                 return method;
             }
@@ -831,8 +869,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @return the method matching the given name and parameters or null
      */
     public MethodNode getMethod(String name, Parameter[] parameters) {
-        for (Object o : getMethods(name)) {
-            MethodNode method = (MethodNode) o;
+        for (MethodNode method : getMethods(name)) {
             if (parametersEqual(method.getParameters(), parameters)) {
                 return method;
             }
@@ -845,6 +882,9 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @return true if this node is derived from the given ClassNode
      */
     public boolean isDerivedFrom(ClassNode type) {
+    	if(this.equals(ClassHelper.VOID_TYPE)) {
+    		return type.equals(ClassHelper.VOID_TYPE) ? true : false;
+    	}
         if (type.equals(ClassHelper.OBJECT_TYPE)) return true;
         ClassNode node = this;
         while (node != null) {
@@ -884,29 +924,19 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      * @param classNode the class node for the interface
      * @return true if this class declares that it implements the given interface
      * or if one of its interfaces extends directly or indirectly the interface
+     *
+     * NOTE: Doesn't consider an interface to implement itself.
+     * I think this is intended to be called on ClassNodes representing
+     * classes, not interfaces.
+     * 
      */
     public boolean declaresInterface(ClassNode classNode) {
         ClassNode[] interfaces = redirect().getInterfaces();
-        if (declaresInterfaceDirect(interfaces, classNode)) return true;
-        List superInterfaces = Arrays.asList(interfaces);
-        while (superInterfaces.size() > 0) {
-            List keep = new ArrayList();
-            for (int i = 0; i < superInterfaces.size(); i++) {
-                ClassNode cn = (ClassNode) superInterfaces.get(i);
-                if (cn.declaresInterface(classNode)) return true;
-                keep.addAll(Arrays.asList(cn.getInterfaces()));
-            }
-            superInterfaces = keep;
+        for (ClassNode cn : interfaces) {
+            if (cn.equals(classNode)) return true;
         }
-        return false;
-    }
-
-    private boolean declaresInterfaceDirect(ClassNode[] interfaces, ClassNode classNode) {
-        int size = interfaces.length;
-        for (int i = 0; i < size; i++) {
-            if (interfaces[i].equals(classNode)) {
-                return true;
-            }
+        for (ClassNode cn : interfaces) {
+            if (cn.declaresInterface(classNode)) return true;
         }
         return false;
     }
@@ -937,6 +967,16 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         superClass = sn;
     }
 
+    public ClassNode [] getUnresolvedInterfaces() {
+        return getUnresolvedInterfaces(true);
+    }
+
+    public ClassNode [] getUnresolvedInterfaces(boolean useRedirect) {
+        if (!useRedirect) return interfaces;
+        if (!redirect().lazyInitDone) redirect().lazyClassInit();
+        return redirect().interfaces;
+    }
+
     public CompileUnit getCompileUnit() {
         if (redirect!=null) return redirect().getCompileUnit();
         if (compileUnit == null && module != null) {
@@ -988,30 +1028,25 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
 
     public void visitContents(GroovyClassVisitor visitor) {
         // now let's visit the contents of the class
-        for (Object o : getProperties()) {
-            PropertyNode pn = (PropertyNode) o;
+        for (PropertyNode pn : getProperties()) {
             visitor.visitProperty(pn);
         }
 
-        for (Object o : getFields()) {
-            FieldNode fn = (FieldNode) o;
+        for (FieldNode fn : getFields()) {
             visitor.visitField(fn);
         }
 
-        for (Object o : getDeclaredConstructors()) {
-            ConstructorNode cn = (ConstructorNode) o;
+        for (ConstructorNode cn : getDeclaredConstructors()) {
             visitor.visitConstructor(cn);
         }
 
-        for (Object o : getMethods()) {
-            MethodNode mn = (MethodNode) o;
+        for (MethodNode mn : getMethods()) {
             visitor.visitMethod(mn);
         }
     }
 
     public MethodNode getGetterMethod(String getterName) {
-        for (Object o : getDeclaredMethods(getterName)) {
-            MethodNode method = (MethodNode) o;
+        for (MethodNode method : getDeclaredMethods(getterName)) {
             if (getterName.equals(method.getName())
                     && ClassHelper.VOID_TYPE!=method.getReturnType()
                     && method.getParameters().length == 0) {
@@ -1024,8 +1059,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     public MethodNode getSetterMethod(String setterName) {
-        for (Object o : getDeclaredMethods(setterName)) {
-            MethodNode method = (MethodNode) o;
+        for (MethodNode method : getDeclaredMethods(setterName)) {
             if (setterName.equals(method.getName())
                     && ClassHelper.VOID_TYPE==method.getReturnType()
                     && method.getParameters().length == 1) {
@@ -1100,9 +1134,8 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         }
         ClassNode node = this;
         do {
-            for (Object o : getMethods(name)) {
-                MethodNode method = (MethodNode) o;
-                if (method.getParameters().length == count) {
+            for (MethodNode method : getMethods(name)) {
+                if (method.getParameters().length == count && !method.isStatic()) {
                     return true;
                 }
             }
@@ -1126,8 +1159,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         ClassNode node = this;
         TupleExpression args = (TupleExpression) arguments;
         do {
-            for (Object o : node.getMethods(name)) {
-                MethodNode method = (MethodNode) o;
+            for (MethodNode method : node.getMethods(name)) {
                 if (method.getParameters().length == count) {
                     boolean match = true;
                     for (int i = 0; i != count; ++i)
@@ -1181,18 +1213,17 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         } else if (arguments instanceof MapExpression) {
         	count = 1;
         }
-
-        for (Object o : getMethods(name)) {
-            MethodNode method = (MethodNode) o;
+        
+        for (MethodNode method : getMethods(name)) {
             if(method.isStatic()) {
-                Parameter[] parameters = method.getParameters();
+                Parameter[] parameters = method.getParameters(); 
                 if (parameters.length == count) return true;
 
                 // handle varargs case
                 if (parameters.length > 0 && parameters[parameters.length - 1].getType().isArray()) {
                     if (count >= parameters.length - 1) return true;
                 }
-
+                
                 // handle parameters with default values
                 int nonDefaultParameters = 0;
                 for(int i = 0; i < parameters.length; i++) {
@@ -1200,7 +1231,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
                 		nonDefaultParameters++;
                 	}
                 }
-
+                
             	if(count < parameters.length && nonDefaultParameters <= count) {
             		return true;
             	}
@@ -1293,40 +1324,48 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
                (getModifiers() & Opcodes.ACC_ANNOTATION)!=0;
     }
 
-    public List getAnnotations() {
+    public List<AnnotationNode> getAnnotations() {
         if (redirect!=null) return redirect.getAnnotations();
         lazyClassInit();
         return super.getAnnotations();
     }
 
-    public List getAnnotations(ClassNode type) {
+    public List<AnnotationNode> getAnnotations(ClassNode type) {
         if (redirect!=null) return redirect.getAnnotations(type);
         lazyClassInit();
         return super.getAnnotations(type);
     }
 
     public void addTransform(Class<? extends ASTTransformation> transform, ASTNode node) {
-        if (transformInstances == null)
-            getTransformInstancesLazy();
-
         GroovyASTTransformation annotation = transform.getAnnotation(GroovyASTTransformation.class);
         Set<ASTNode> nodes = transformInstances.get(annotation.phase()).get(transform);
         if (nodes == null) {
-            nodes = new LinkedHashSet();
+            nodes = new LinkedHashSet<ASTNode>();
             transformInstances.get(annotation.phase()).put(transform, nodes);
         }
         nodes.add(node);
     }
 
     public Map<Class <? extends ASTTransformation>, Set<ASTNode>> getTransforms(CompilePhase phase) {
-        if (transformInstances == null)
-            return Collections.EMPTY_MAP;
-
         return transformInstances.get(phase);
     }
 
     public void renameField(String oldName, String newName) {
-        final Map index = redirect().getFieldIndexLazy();
+        ClassNode r = redirect ();
+        if (r.fieldIndex == null)
+            r.fieldIndex = new HashMap<String,FieldNode> ();
+        final Map<String,FieldNode> index = r.fieldIndex;
         index.put(newName, index.remove(oldName));
     }
+    
+    public boolean isEnum() {
+        return (getModifiers()&Opcodes.ACC_ENUM) != 0;
+     }
+
+    /**
+     * @return iterator of inner classes defined inside this one
+     */
+    public Iterator<InnerClassNode> getInnerClasses() {
+        return (innerClasses == null ? Collections.<InnerClassNode>emptyList() : innerClasses).iterator();
+    }
 }
