@@ -25,7 +25,7 @@ import java.util.List;
  * Represents a method declaration
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 11961 $
+ * @version $Revision: 18462 $
  */
 public class MethodNode extends AnnotatedNode implements Opcodes {
 
@@ -38,9 +38,14 @@ public class MethodNode extends AnnotatedNode implements Opcodes {
     private boolean dynamicReturnType;
     private VariableScope variableScope;
     private final ClassNode[] exceptions;
+    private final boolean staticConstructor;
     
     // type spec for generics
     private GenericsType[] genericsTypes=null;
+    private boolean hasDefault;
+    
+    // cached data
+    String typeDescriptor;
 
     public MethodNode(String name, int modifiers, ClassNode returnType, Parameter[] parameters, ClassNode[] exceptions, Statement code) {
         this.name = name;
@@ -50,34 +55,38 @@ public class MethodNode extends AnnotatedNode implements Opcodes {
         VariableScope scope = new VariableScope();
         setVariableScope(scope);
         setParameters(parameters);
-        
+        this.hasDefault = false;       
         this.exceptions = exceptions;
+        this.staticConstructor = (name != null && name.equals("<clinit>")) ? true : false;
     }
 
     /**
      * The type descriptor for a method node is a string containing the name of the method, its return type,
      * and its parameter types in a canonical form. For simplicity, I'm using the format of a Java declaration
-     * without parameter names, and with $dynamic as the type for any dynamically typed values.
+     * without parameter names.
      */
-    // TODO: add test case for type descriptor
     public String getTypeDescriptor() {
-        StringBuffer buf = new StringBuffer(name.length()+parameters.length*10);
-        // buf.append(dynamicReturnType ? "$dynamic" : cleanupTypeName(returnType));
-        //
-        buf.append(returnType.getName()); // br  to replace the above. Dynamic type returns Object.
-        //
-        buf.append(' ');
-        buf.append(name);
-        buf.append('(');
-        for (int i = 0; i < parameters.length; i++) {
-            if (i > 0) {
-                buf.append(", ");
+        if (typeDescriptor==null) { 
+            StringBuffer buf = new StringBuffer(name.length()+parameters.length*10);
+            buf.append(returnType.getName());
+            buf.append(' ');
+            buf.append(name);
+            buf.append('(');
+            for (int i = 0; i < parameters.length; i++) {
+                if (i > 0) {
+                    buf.append(", ");
+                }
+                Parameter param = parameters[i];
+                buf.append(param.getType().getName());
             }
-            Parameter param = parameters[i];
-            buf.append(param.getType().getName());
+            buf.append(')');
+            typeDescriptor = buf.toString();
         }
-        buf.append(')');
-        return buf.toString();
+        return typeDescriptor;
+    }
+    
+    private void invalidateCachedData() {
+        typeDescriptor = null;
     }
  
     public boolean isVoidMethod() {
@@ -97,6 +106,7 @@ public class MethodNode extends AnnotatedNode implements Opcodes {
     }
 
     public void setModifiers(int modifiers) {
+        invalidateCachedData();
         this.modifiers = modifiers;
     }
 
@@ -109,6 +119,7 @@ public class MethodNode extends AnnotatedNode implements Opcodes {
     }
 
     public void setParameters(Parameter[] parameters) {
+        invalidateCachedData();
         VariableScope scope = new VariableScope();
         this.parameters = parameters;
         if (parameters != null && parameters.length > 0) {
@@ -153,6 +164,10 @@ public class MethodNode extends AnnotatedNode implements Opcodes {
         return (modifiers & ACC_PUBLIC) != 0;
     }
 
+    public boolean isPrivate() {
+        return (modifiers & ACC_PRIVATE) != 0;
+    }
+
     public boolean isProtected() {
         return (modifiers & ACC_PROTECTED) != 0;
     }
@@ -162,10 +177,11 @@ public class MethodNode extends AnnotatedNode implements Opcodes {
     }
 
     public String toString() {
-        return super.toString() + "[name: " + name + "]";
+        return "MethodNode@"+hashCode()+"[" + getTypeDescriptor() + "]";
     }
 
     public void setReturnType(ClassNode returnType) {
+        invalidateCachedData();
     	dynamicReturnType |= ClassHelper.DYNAMIC_TYPE==returnType;
         this.returnType = returnType;
         if (returnType==null) this.returnType = ClassHelper.OBJECT_TYPE;
@@ -179,11 +195,11 @@ public class MethodNode extends AnnotatedNode implements Opcodes {
         if (code == null) return null;
         Statement first = code;
         while (first instanceof BlockStatement) {
-            List list = ((BlockStatement) first).getStatements();
+            List<Statement> list = ((BlockStatement) first).getStatements();
             if (list.isEmpty()) {
                 first=null;
             } else {
-                first = (Statement) list.get(0);
+                first = list.get(0);
             }
         }
         return first;
@@ -194,6 +210,19 @@ public class MethodNode extends AnnotatedNode implements Opcodes {
     }
 
     public void setGenericsTypes(GenericsType[] genericsTypes) {
+        invalidateCachedData();
         this.genericsTypes = genericsTypes;
     }
+
+    public void setAnnotationDefault(boolean b) {
+        this.hasDefault = b;
+    }
+
+    public boolean hasAnnotationDefault() {
+        return hasDefault;
+    }
+
+    public boolean isStaticConstructor() {
+        return staticConstructor;
+    }
 }
