@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2009 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,12 +22,12 @@ import org.codehaus.groovy.ast.expr.*;
 import org.codehaus.groovy.ast.stmt.BlockStatement;
 import org.codehaus.groovy.ast.stmt.CatchStatement;
 import org.codehaus.groovy.ast.stmt.ForStatement;
+import org.codehaus.groovy.ast.stmt.Statement;
 import org.codehaus.groovy.control.SourceUnit;
+import org.codehaus.groovy.syntax.Types;
+import org.objectweb.asm.Opcodes;
 
-import java.util.Iterator;
 import java.util.LinkedList;
-import java.util.List;
-import java.util.Map;
 
 /**
  * goes through an AST and initializes the scopes
@@ -35,9 +35,9 @@ import java.util.Map;
  * @author Jochen Theodorou
  */
 public class VariableScopeVisitor extends ClassCodeVisitorSupport {
-    
+
     private static final Expression CALL = new ConstantExpression("call");
-    
+
     private VariableScope currentScope = null;
     private VariableScope headScope = new VariableScope();
     private ClassNode currentClass = null;
@@ -45,6 +45,8 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     private boolean inClosure = false;
     private boolean inPropertyExpression = false;
     private boolean isSpecialConstructorCall = false;
+    private boolean inConstructor = false;
+    private boolean inStaticConstructor = false;
 
     private LinkedList stateStack = new LinkedList();
 
@@ -52,11 +54,13 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         VariableScope scope;
         ClassNode clazz;
         boolean closure;
+        boolean inConstructor; 
 
         StateStackElement() {
             scope = VariableScopeVisitor.this.currentScope;
             clazz = VariableScopeVisitor.this.currentClass;
             closure = VariableScopeVisitor.this.inClosure;
+            inConstructor = VariableScopeVisitor.this.inConstructor;
         }
     }
 
@@ -66,7 +70,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     }
 
     // ------------------------------
-    // helper methods   
+    // helper methods
     //------------------------------
 
     private void pushState(boolean isStatic) {
@@ -92,14 +96,15 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         currentScope = element.scope;
         currentClass = element.clazz;
         inClosure = element.closure;
+        inConstructor = element.inConstructor;
     }
 
     private void declare(Parameter[] parameters, ASTNode node) {
-        for (int i = 0; i < parameters.length; i++) {
-            if (parameters[i].hasInitialExpression()) {
-                parameters[i].getInitialExpression().visit(this);
+        for (Parameter parameter : parameters) {
+            if (parameter.hasInitialExpression()) {
+                parameter.getInitialExpression().visit(this);
             }
-            declare(parameters[i], node);
+            declare(parameter, node);
         }
     }
 
@@ -156,27 +161,19 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         if (cn.isScript()) {
             return new DynamicVariable(name, false);
         }
-        List l = cn.getFields();
-        for (Iterator iter = l.iterator(); iter.hasNext();) {
-            FieldNode f = (FieldNode) iter.next();
-            if (f.getName().equals(name)) return f;
+
+        for (FieldNode fn : cn.getFields()) {
+            if (fn.getName().equals(name)) return fn;
         }
 
-        l = cn.getMethods();
-        for (Iterator iter = l.iterator(); iter.hasNext();) {
-            MethodNode f = (MethodNode) iter.next();
-            String methodName = f.getName();
-            String pName = getPropertyName(f);
-            if (pName == null) continue;
-            if (!pName.equals(name)) continue;
-            PropertyNode var = new PropertyNode(pName, f.getModifiers(), getPropertyType(f), cn, null, null, null);
-            return var;
+        for (MethodNode mn : cn.getMethods()) {
+            String pName = getPropertyName(mn);
+            if (pName != null && pName.equals(name))
+                return new PropertyNode(pName, mn.getModifiers(), getPropertyType(mn), cn, null, null, null);
         }
 
-        l = cn.getProperties();
-        for (Iterator iter = l.iterator(); iter.hasNext();) {
-            PropertyNode f = (PropertyNode) iter.next();
-            if (f.getName().equals(name)) return f;
+        for (PropertyNode pn : cn.getProperties()) {
+            if (pn.getName().equals(name)) return pn;
         }
 
         Variable ret = findClassMember(cn.getSuperClass(), name);
@@ -185,7 +182,6 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     }
 
     private ClassNode getPropertyType(MethodNode m) {
-        String name = m.getName();
         if (m.getReturnType() != ClassHelper.VOID_TYPE) {
             return m.getReturnType();
         }
@@ -199,7 +195,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         if (pname.length() == 0) return null;
         pname = java.beans.Introspector.decapitalize(pname);
 
-        if (name.startsWith("get") && m.getReturnType() == ClassHelper.VOID_TYPE) {
+        if (name.startsWith("get") && (m.getReturnType() == ClassHelper.VOID_TYPE || m.getParameters().length != 0)) {
             return null;
         }
         if (name.startsWith("set") && m.getParameters().length != 1) {
@@ -209,7 +205,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     }
 
     // -------------------------------
-    // different Variable based checks  
+    // different Variable based checks
     // -------------------------------
 
     private Variable checkVariableNameForDeclaration(String name, Expression expression) {
@@ -217,14 +213,8 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
 
         VariableScope scope = currentScope;
         Variable var = new DynamicVariable(name, currentScope.isInStaticContext());
-        Variable dummyStart = var;
         // try to find a declaration of a variable
-        VariableScope dynamicScope = null;
-        while (!scope.isRoot()) {
-            if (dynamicScope == null && scope.isResolvingDynamic()) {
-                dynamicScope = scope;
-            }
-
+        while (true) {
             Variable var1;
             var1 = scope.getDeclaredVariable(var.getName());
 
@@ -233,7 +223,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
                 break;
             }
 
-            var1 = (Variable) scope.getReferencedLocalVariable(var.getName());
+            var1 = scope.getReferencedLocalVariable(var.getName());
             if (var1 != null) {
                 var = var1;
                 break;
@@ -249,24 +239,12 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
             if (classScope != null) {
                 Variable member = findClassMember(classScope, var.getName());
                 if (member != null) {
-                    boolean cc = currentScope.isInStaticContext() || isSpecialConstructorCall;
-                    boolean cm = member.isInStaticContext();
-                    //
-                    // we don't allow access from dynamic context to static context
-                    //
-                    // cm==cc: 
-                    //   we always allow access if the context is in both cases static 
-                    //   or dynamic
-                    // cm==true: 
-                    //   the member is static, which means access is always allowed
-                    // cm||cm==cc:
-                    //   is false only for the case cc==true and cm==false, which means
-                    //   the member is a dynamic context, but the current scope is static.
-                    //
-                    // One example for (cm||cm==cc)==false is a static method trying to 
-                    // access a non static field.
-                    //
-                    if (cm || cm == cc) var = member;
+                    boolean staticScope = currentScope.isInStaticContext() || isSpecialConstructorCall;
+                    boolean staticMember = member.isInStaticContext();
+                    // We don't allow a static context (e.g. a static method) to access
+                    // a non-static variable (e.g. a non-static field).
+                    if (! (staticScope && ! staticMember))
+                        var = member;
                 }
                 break;
             }
@@ -275,35 +253,16 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
 
         VariableScope end = scope;
 
-        if (scope.isRoot() && dynamicScope == null) {
-            // no matching scope found
-            declare(var, expression);
-            addError("The variable " + var.getName() +
-                    " is undefined in the current scope", expression);
-        } else if (scope.isRoot() && dynamicScope != null) {
-            // no matching scope found, but there was a scope that
-            // resolves dynamic
-            scope = dynamicScope;
-        }
-
-        if (!scope.isRoot()) {
-            scope = currentScope;
-            while (scope != end) {
-                Map references = null;
-                if (end.isClassScope() || end.isRoot() ||
-                        (end.isReferencedClassVariable(name) && end.getDeclaredVariable(name) == null)) {
-                    scope.putReferencedClassVariable(var);
-                } else {
-                    var.setClosureSharedVariable(var.isClosureSharedVariable() || inClosure);
-                    scope.putReferencedLocalVariable(var);
-                }
-                scope = scope.getParent();
-            }
-            if (end.isResolvingDynamic()) {
-                if (end.getDeclaredVariable(var.getName()) == null) {
-                    end.putDeclaredVariable(var);
-                }
+        scope = currentScope;
+        while (scope != end) {
+            if (end.isClassScope() ||
+                    (end.isReferencedClassVariable(name) && end.getDeclaredVariable(name) == null)) {
+                scope.putReferencedClassVariable(var);
+            } else {
+                var.setClosureSharedVariable(var.isClosureSharedVariable() || inClosure);
+                scope.putReferencedLocalVariable(var);
             }
+            scope = scope.getParent();
         }
 
         return var;
@@ -313,6 +272,8 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
      * a property on "this", like this.x is transformed to a
      * direct field access, so we need to check the
      * static context here
+     *
+     * @param pe the property expression to check
      */
     private void checkPropertyOnExplicitThis(PropertyExpression pe) {
         if (!currentScope.isInStaticContext()) return;
@@ -321,6 +282,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         VariableExpression ve = (VariableExpression) object;
         if (!ve.getName().equals("this")) return;
         String name = pe.getPropertyAsString();
+        if (name.equals("class")) return;
         if (name == null) return;
         Variable member = findClassMember(currentClass, name);
         if (member == null) return;
@@ -341,7 +303,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     }
 
     // ------------------------------
-    // code visit  
+    // code visit
     // ------------------------------
 
     public void visitBlockStatement(BlockStatement block) {
@@ -354,7 +316,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     public void visitForLoop(ForStatement forLoop) {
         pushState();
         forLoop.setVariableScope(currentScope);
-        Parameter p = (Parameter) forLoop.getVariable();
+        Parameter p = forLoop.getVariable();
         p.setInStaticContext(currentScope.isInStaticContext());
         if (p != ForStatement.FOR_LOOP_DUMMY) declare(p, forLoop);
         super.visitForLoop(forLoop);
@@ -362,20 +324,20 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     }
 
     public void visitDeclarationExpression(DeclarationExpression expression) {
-        // visit right side first to avoid the usage of a 
+        // visit right side first to avoid the usage of a
         // variable before its declaration
         expression.getRightExpression().visit(this);
-        
+
         // no need to visit left side, just get the variable name
         if (expression.isMultipleAssignmentDeclaration()) {
             ArgumentListExpression list = (ArgumentListExpression) expression.getLeftExpression();
-            for (Iterator it=list.getExpressions().iterator(); it.hasNext();) {
-                VariableExpression exp = (VariableExpression) it.next();
+            for (Expression e : list.getExpressions()) {
+                VariableExpression exp = (VariableExpression) e;
                 declare(exp);
             }
         } else {
-            declare(expression.getVariableExpression());           
-        }        
+            declare(expression.getVariableExpression());
+        }
     }
 
     public void visitVariableExpression(VariableExpression expression) {
@@ -400,23 +362,20 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         pushState();
 
         inClosure = true;
-        // as result of the Paris meeting Closure resolves
-        // always dynamically
-        currentScope.setDynamicResolving(true);
 
         expression.setVariableScope(currentScope);
 
         if (expression.isParameterSpecified()) {
             Parameter[] parameters = expression.getParameters();
-            for (int i = 0; i < parameters.length; i++) {
-                parameters[i].setInStaticContext(currentScope.isInStaticContext());
-                if (parameters[i].hasInitialExpression()) {
-                    parameters[i].getInitialExpression().visit(this);
+            for (Parameter parameter : parameters) {
+                parameter.setInStaticContext(currentScope.isInStaticContext());
+                if (parameter.hasInitialExpression()) {
+                    parameter.getInitialExpression().visit(this);
                 }
-                declare(parameters[i], expression);                
+                declare(parameter, expression);
             }
         } else if (expression.getParameters() != null) {
-            Parameter var = new Parameter(ClassHelper.OBJECT_TYPE,"it");
+            Parameter var = new Parameter(ClassHelper.OBJECT_TYPE, "it");
             var.setInStaticContext(currentScope.isInStaticContext());
             currentScope.putDeclaredVariable(var);
         }
@@ -427,7 +386,7 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
 
     public void visitCatchStatement(CatchStatement statement) {
         pushState();
-        Parameter p = (Parameter) statement.getVariable();
+        Parameter p = statement.getVariable();
         p.setInStaticContext(currentScope.isInStaticContext());
         declare(p, statement);
         super.visitCatchStatement(statement);
@@ -442,15 +401,13 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
     }
 
     // ------------------------------
-    // class visit  
+    // class visit
     // ------------------------------
 
     public void visitClass(ClassNode node) {
         pushState();
 
         currentClass = node;
-        boolean dynamicMode = node.isScript();
-        currentScope.setDynamicResolving(dynamicMode);
         currentScope.setClassScope(node);
 
         super.visitClass(node);
@@ -459,13 +416,13 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
 
     protected void visitConstructorOrMethod(MethodNode node, boolean isConstructor) {
         pushState(node.isStatic());
-
+        inConstructor = isConstructor;
+        inStaticConstructor = node.isStaticConstructor();
         node.setVariableScope(currentScope);
-
+        
         // GROOVY-2156
         Parameter[] parameters = node.getParameters();
-        for (int i = 0; i < parameters.length; i++) {
-            Parameter parameter = parameters[i];
+        for (Parameter parameter : parameters) {
             visitAnnotations(parameter);
         }
 
@@ -479,11 +436,11 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         if (call.isImplicitThis() && call.getMethod() instanceof ConstantExpression) {
             ConstantExpression methodNameConstant = (ConstantExpression) call.getMethod();
             Object value = methodNameConstant.getText();
-            
+
             if (!(value instanceof String)) {
                 throw new GroovyBugError("tried to make a method call with a non-String constant method name.");
             }
-            
+
             String methodName = (String) value;
             Variable v = checkVariableNameForDeclaration(methodName, call);
             if (v != null && !(v instanceof DynamicVariable)) {
@@ -505,6 +462,81 @@ public class VariableScopeVisitor extends ClassCodeVisitorSupport {
         isSpecialConstructorCall = call.isSpecialCall();
         super.visitConstructorCallExpression(call);
         isSpecialConstructorCall = false;
+        if (!call.isUsingAnnonymousInnerClass()) return;
+
+        pushState();
+        InnerClassNode innerClass = (InnerClassNode) call.getType();
+        innerClass.setVariableScope(currentScope);
+        for (MethodNode method : innerClass.getMethods()) {
+            Parameter[] parameters = method.getParameters();
+            if (parameters.length == 0) parameters = null; // null means no implicit "it"
+            ClosureExpression cl = new ClosureExpression(parameters, method.getCode());
+            visitClosureExpression(cl);
+        }
+
+        boolean ic = inClosure;
+        inClosure = true;
+        for (FieldNode field : innerClass.getFields()) {
+            final Expression expression = field.getInitialExpression();
+            if (expression != null) {
+                expression.visit(this);
+            }
+        }
+
+        for (Statement statement : innerClass.getObjectInitializerStatements()) {
+            statement.visit(this);
+        }
+        inClosure = ic;
+        popState();
+    }
+    
+    public void visitBinaryExpression(BinaryExpression expression) {
+        super.visitBinaryExpression(expression);
+        switch (expression.getOperation().getType()){
+            case Types.EQUAL: // = assignment
+            case Types.BITWISE_AND_EQUAL:
+            case Types.BITWISE_OR_EQUAL:
+            case Types.BITWISE_XOR_EQUAL:
+            case Types.PLUS_EQUAL:
+            case Types.MINUS_EQUAL:
+            case Types.MULTIPLY_EQUAL:
+            case Types.DIVIDE_EQUAL:
+            case Types.INTDIV_EQUAL:
+            case Types.MOD_EQUAL:
+            case Types.POWER_EQUAL:
+            case Types.LEFT_SHIFT_EQUAL:
+            case Types.RIGHT_SHIFT_EQUAL:
+            case Types.RIGHT_SHIFT_UNSIGNED_EQUAL:
+                checkFinalFieldAccess(expression.getLeftExpression());
+                break;
+            default: break;
+        }
+        
+    }
+
+    private void checkFinalFieldAccess(Expression expression) {
+        if (!(expression instanceof VariableExpression)) return;
+        boolean isFinal, isStatic, error;
+        int modifiers = 0;
+        VariableExpression ve = (VariableExpression) expression;
+        Variable v = ve.getAccessedVariable();
+        if (v instanceof PropertyNode) {
+            PropertyNode pn = (PropertyNode) v;
+            modifiers = pn.getModifiers();
+        } else if (v instanceof FieldNode) {
+            FieldNode fn = (FieldNode) v;
+            modifiers = fn.getModifiers();
+        }
+        /*
+         *  if it is static final but not accessed inside a static constructor, or,
+         *  if it is an instance final but not accessed inside a instance constructor, it is an error
+         */
+        isFinal = (modifiers & Opcodes.ACC_FINAL) != 0;
+        isStatic = (modifiers & Opcodes.ACC_STATIC) != 0;
+        error = isFinal && ((isStatic && !inStaticConstructor) || (!isStatic && !inConstructor));
+
+        if (error) addError("cannnot access final field or property " +
+                            "outside of constructor.", expression);
     }
 
     public void visitProperty(PropertyNode node) {
