@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2009 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,6 +30,8 @@ import org.objectweb.asm.Opcodes;
 
 import java.io.File;
 import java.io.IOException;
+import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLConnection;
@@ -62,7 +64,8 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private boolean inClosure = false;
     private boolean isSpecialConstructorCall = false;
 
-    private Map genericParameterNames = new HashMap();
+    private Map<String, GenericsType> genericParameterNames = new HashMap<String, GenericsType>();
+    private Set<FieldNode> fieldTypesChecked = new HashSet<FieldNode>();
 
     /**
      * we use ConstructedClassWithPackage to limit the resolving the compiler
@@ -144,21 +147,19 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     protected void visitConstructorOrMethod(MethodNode node, boolean isConstructor) {
         VariableScope oldScope = currentScope;
         currentScope = node.getVariableScope();
-        Map oldPNames = genericParameterNames;
-        genericParameterNames = new HashMap(genericParameterNames);
+        Map<String, GenericsType> oldPNames = genericParameterNames;
+        genericParameterNames = new HashMap<String, GenericsType>(genericParameterNames);
 
         resolveGenericsHeader(node.getGenericsTypes());
 
         Parameter[] paras = node.getParameters();
-        for (int i = 0; i < paras.length; i++) {
-            Parameter p = paras[i];
+        for (Parameter p : paras) {
             p.setInitialExpression(transform(p.getInitialExpression()));
             resolveOrFail(p.getType(), p.getType());
             visitAnnotations(p);
         }
         ClassNode[] exceptions = node.getExceptions();
-        for (int i = 0; i < exceptions.length; i++) {
-            ClassNode t = exceptions[i];
+        for (ClassNode t : exceptions) {
             resolveOrFail(t, node);
         }
         resolveOrFail(node.getReturnType(), node);
@@ -171,7 +172,9 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
 
     public void visitField(FieldNode node) {
         ClassNode t = node.getType();
-        resolveOrFail(t, node);
+        if(!fieldTypesChecked.contains(node)) {
+            resolveOrFail(t, node);
+        }
         super.visitField(node);
     }
 
@@ -179,6 +182,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         ClassNode t = node.getType();
         resolveOrFail(t, node);
         super.visitProperty(node);
+        fieldTypesChecked.add(node.getField());
     }
 
     private boolean resolveToInner (ClassNode type) {
@@ -200,9 +204,9 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         type.setName(saved);
         return false;
     }
-    
+
     private boolean resolveToInnerEnum (ClassNode type) {
-        // GROOVY-3483: It may be an inner enum defined by this class itself, in which case it does not need to be
+        // GROOVY-3110: It may be an inner enum defined by this class itself, in which case it does not need to be
         // explicitly qualified by the currentClass name
     	String name = type.getName();
         if(currentClass != type && !name.contains(".") && type.getClass().equals(ClassNode.class)) {
@@ -233,8 +237,8 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     }
 
     private boolean resolve(ClassNode type, boolean testModuleImports, boolean testDefaultImports, boolean testStaticInnerClasses) {
-        if (type.isResolved() || type.isPrimaryClassNode()) return true;
         resolveGenericsTypes(type.getGenericsTypes());
+        if (type.isResolved() || type.isPrimaryClassNode()) return true;
         if (type.isArray()) {
             ClassNode element = type.getComponentType();
             boolean resolved = resolve(element, testModuleImports, testDefaultImports, testStaticInnerClasses);
@@ -249,7 +253,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         if (currentClass == type) return true;
 
         if (genericParameterNames.get(type.getName()) != null) {
-            GenericsType gt = (GenericsType) genericParameterNames.get(type.getName());
+            GenericsType gt = genericParameterNames.get(type.getName());
             type.setRedirect(gt.getType());
             type.setGenericsTypes(new GenericsType[]{gt});
             type.setGenericsPlaceHolder(true);
@@ -261,23 +265,71 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             return true;
         }
 
-        return  resolveFromModule(type, testModuleImports) ||
+        return  resolveNestedClass(type) ||
+                resolveFromModule(type, testModuleImports) ||
                 resolveFromCompileUnit(type) ||
                 resolveFromDefaultImports(type, testDefaultImports) ||
                 resolveFromStaticInnerClasses(type, testStaticInnerClasses) ||
                 resolveFromClassCache(type) ||
                 resolveToClass(type) ||
                 resolveToScript(type);
-
     }
-
+    
+    private boolean resolveNestedClass(ClassNode type) {
+        // we have for example a class name A, are in class X
+        // and there is a nested class A$X. we want to be able 
+        // to access that class directly, so A becomes a valid
+        // name in X.
+        String name = currentClass.getName()+"$"+type.getName();
+        ClassNode val = ClassHelper.make(name);
+        if (resolveFromCompileUnit(val)) {
+            type.setRedirect(val);
+            return true;
+        }
+        
+        // another case we want to check here is if we are in a
+        // nested class A$B$C and want to access B without
+        // qualifying it by A.B. A alone will work, since that
+        // is the qualified (minus package) name of that class
+        // anyway. 
+        
+        // That means if the current class is not an InnerClassNode
+        // there is nothing to be done.
+        if (!(currentClass instanceof InnerClassNode)) return false;
+        
+        // since we have B and want to get A we start with the most 
+        // outer class, put them together and then see if that does
+        // already exist. In case of B from within A$B we are done 
+        // after the first step already. In case of for example
+        // A.B.C.D.E.F and accessing E from F we test A$E=failed, 
+        // A$B$E=failed, A$B$C$E=fail, A$B$C$D$E=success
+        
+        LinkedList<ClassNode> outerClasses = new LinkedList<ClassNode>();
+        ClassNode outer = currentClass.getOuterClass();
+        while (outer!=null) {
+            outerClasses.addFirst(outer);
+            outer = outer.getOuterClass();
+        }
+        // most outer class is now element 0
+        for (ClassNode testNode : outerClasses) {
+            name = testNode.getName()+"$"+type.getName();
+            val = ClassHelper.make(name);
+            if (resolveFromCompileUnit(val)) {
+                type.setRedirect(val);
+                return true;
+            }
+        }        
+        
+        return false;   
+    }
+    
     private boolean resolveFromClassCache(ClassNode type) {
         String name = type.getName();
         Object val = cachedClasses.get(name);
         if (val == null || val == NO_CLASS) {
             return false;
         } else {
-            setClass(type, (Class) val);
+            type.setRedirect((ClassNode)val);
             return true;
         }
     }
@@ -378,7 +430,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             if (type instanceof ConstructedClassWithPackage) {
                 // we replace '.' only in the className part
                 // with '$' to find an inner class. The case that
-                // the packageis really a class is handled else where
+                // the package is really a class is handled else where
                 ConstructedClassWithPackage tmp = (ConstructedClassWithPackage) type;
                 String name = ((ConstructedClassWithPackage) type).className;
                 tmp.className = replaceLastPoint(name);
@@ -445,11 +497,6 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         return false;
     }
 
-    private void setClass(ClassNode n, Class cls) {
-        ClassNode cn = ClassHelper.make(cls);
-        n.setRedirect(cn);
-    }
-
     private void ambiguousClass(ClassNode type, ClassNode iType, String name) {
         if (type.getName().equals(iType.getName())) {
             addError("reference to " + name + " is ambiguous, both class " + type.getName() + " and " + iType.getName() + " match", type);
@@ -480,7 +527,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
          */
         while (true) {
             pname = name.substring(0, index);
-            ClassNode aliasedNode = module.getImport(pname);
+            ClassNode aliasedNode = module.getImportType(pname);
 
             if (aliasedNode != null) {
                 if (pname.length() == name.length()) {
@@ -541,9 +588,8 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             newNameUsed = true;
         }
         // look into the module node if there is a class with that name
-        List moduleClasses = module.getClasses();
-        for (Iterator iter = moduleClasses.iterator(); iter.hasNext();) {
-            ClassNode mClass = (ClassNode) iter.next();
+        List<ClassNode> moduleClasses = module.getClasses();
+        for (ClassNode mClass : moduleClasses) {
             if (mClass.getName().equals(type.getName())) {
                 if (mClass != type) type.setRedirect(mClass);
                 return true;
@@ -566,14 +612,13 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             }
 
             // check module node imports packages
-            List packages = module.getImportPackages();
-            for (Iterator iter = packages.iterator(); iter.hasNext();) {
-                String packagePrefix = (String) iter.next();
+            for (ImportNode importNode : module.getStarImports()) {
+                String packagePrefix = importNode.getPackageName();
                 // We limit the inner class lookups here by using ConstructedClassWithPackage.
                 // This way only the name will change, the packagePrefix will
                 // not be included in the lookup. The case where the
                 // packagePrefix is really a class is handled else where.
-                ConstructedClassWithPackage tmp =  new ConstructedClassWithPackage(packagePrefix,name);
+                ConstructedClassWithPackage tmp = new ConstructedClassWithPackage(packagePrefix, name);
                 if (resolve(tmp, false, false, true)) {
                     ambiguousClass(type, tmp, name);
                     type.setRedirect(tmp.redirect());
@@ -596,7 +641,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             cachedClasses.put(name,NO_CLASS);
         }
 
-        // We use here the class cahce cachedClasses to prevent
+        // We use here the class cache cachedClasses to prevent
         // calls to ClassLoader#loadClass. disabling this cache will
         // cause a major performance hit. Unlike at the end of this
         // method we do not return true or false depending on if we
@@ -633,8 +678,9 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             return false;
         }*/
         if (cls == null) return false;
-        cachedClasses.put(name, cls);
-        setClass(type, cls);
+        ClassNode cn = ClassHelper.make(cls);
+        cachedClasses.put(name, cn);
+        type.setRedirect(cn);
         //NOTE: we might return false here even if we found a class,
         //      because  we want to give a possible script a chance to
         //      recompile. This can only be done if the loader was not
@@ -691,16 +737,16 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                     // can still be resolved to the class foo.Bar and the static
                     // field bar.
                     if (!testVanillaNameForClass(varName)) return null;
-                    doInitialClassTest= false;
+                    doInitialClassTest = false;
                     name = varName;
                 } else {
                     name = varName + "." + name;
                 }
                 break;
             }
-            // anything other than PropertyExpressions, ClassExpression or
+            // anything other than PropertyExpressions or
             // VariableExpressions will stop resolving
-            else if (!(it.getClass() == PropertyExpression.class)) {
+            else if (it.getClass() != PropertyExpression.class) {
                 return null;
             } else {
                 PropertyExpression current = (PropertyExpression) it;
@@ -733,7 +779,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     // a PropertyExpression with the ClassExpression of Integer as objectExpression
     // and class as property
     private Expression correctClassClassChain(PropertyExpression pe) {
-        LinkedList stack = new LinkedList();
+        LinkedList<Expression> stack = new LinkedList<Expression>();
         ClassExpression found = null;
         for (Expression it = pe; it != null; it = ((PropertyExpression) it).getObjectExpression()) {
             if (it instanceof ClassExpression) {
@@ -769,7 +815,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
 
         Expression objectExpression = pe.getObjectExpression();
         inPropertyExpression = true;
-        isTopLevelProperty = !(objectExpression.getClass() == PropertyExpression.class);
+        isTopLevelProperty = (objectExpression.getClass() != PropertyExpression.class);
         objectExpression = transform(objectExpression);
         // we handle the property part as if it were not part of the property
         inPropertyExpression = false;
@@ -803,9 +849,50 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             }
         }
         Expression ret = pe;
+        checkThisAndSuperAsPropertyAccess(pe);
         if (isTopLevelProperty) ret = correctClassClassChain(pe);
         return ret;
     }
+    
+    
+    private void checkThisAndSuperAsPropertyAccess(PropertyExpression expression) {
+        if (expression.isImplicitThis()) return;
+        String prop = expression.getPropertyAsString();
+        if (prop==null) return;
+        if (!prop.equals("this") && !prop.equals("super")) return;
+        
+        //TODO: add support for super
+        if (prop.equals("super")) {
+            addError("Inner classes referencing outer classes using super is not supported yet.",expression);
+        }
+        
+        if (!(expression.getObjectExpression() instanceof ClassExpression)) {
+            addError("The usage of '.this' or '.super' requires an explicit class in front.",expression);
+            return;
+        }
+            
+        if (!(currentClass instanceof InnerClassNode)) {
+            addError("The usage of '.this' and '.super' is only allowed in a inner class",expression);
+            return;
+        }
+
+        
+        ClassNode type  = expression.getObjectExpression().getType();
+        ClassNode iterType = currentClass;
+        while (iterType!=null) {
+            if (iterType.equals(type)) break;
+            iterType = iterType.getOuterClass();
+        }
+        if (iterType==null) {
+            addError("The class '"+type.getName()+"' needs to be an "+
+                     "outer class of '"+currentClass.getName()+"'.",expression);
+        }
+
+        
+        if ((currentClass.getModifiers() & Opcodes.ACC_STATIC)==0) return;
+        if (!currentScope.isInStaticContext()) return;
+        addError("The usage of '.this' and '.super' is only in nonstatic context",expression);
+    }
 
     protected Expression transformVariableExpression(VariableExpression ve) {
         Variable v = ve.getAccessedVariable();
@@ -819,7 +906,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             if (!isClass) {
                 // It was no primitive type, so next we see if the name,
                 // which is a vanilla name, starts with a lower case letter.
-                // In that case we change t to a LowerCaseClass to let the
+                // In that case we change it to a LowerCaseClass to let the
                 // compiler skip the resolving at several places in this class.
                 if (Character.isLowerCase(name.charAt(0))) {
                   t = new LowerCaseClass(name);
@@ -881,14 +968,14 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         inClosure = true;
         Parameter[] paras = ce.getParameters();
         if (paras != null) {
-            for (int i = 0; i < paras.length; i++) {
-                ClassNode t = paras[i].getType();
+            for (Parameter para : paras) {
+                ClassNode t = para.getType();
                 resolveOrFail(t, ce);
-                if(paras[i].hasInitialExpression()) {
-                	Object initialVal = paras[i].getInitialExpression();
-                	if(initialVal instanceof Expression) {
-                		transform((Expression)initialVal);
-                	}
+                if (para.hasInitialExpression()) {
+                    Object initialVal = para.getInitialExpression();
+                    if (initialVal instanceof Expression) {
+                        transform((Expression) initialVal);
+                    }
                 }
             }
         }
@@ -919,7 +1006,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         result.setSourcePosition(mce);
         return result;
     }
-
+    
     protected Expression transformDeclarationExpression(DeclarationExpression de) {
         Expression oldLeft = de.getLeftExpression();
         Expression left = transform(oldLeft);
@@ -939,37 +1026,91 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         AnnotationNode an = (AnnotationNode) ace.getValue();
         ClassNode type = an.getClassNode();
         resolveOrFail(type, ", unable to find class for annotation", an);
-        for (Iterator iter = an.getMembers().entrySet().iterator(); iter.hasNext();) {
-            Map.Entry member = (Map.Entry) iter.next();
-            Expression memberValue = (Expression) member.getValue();
-            member.setValue(transform(memberValue));
+        for (Map.Entry<String, Expression> member : an.getMembers().entrySet()) {
+            member.setValue(transform(member.getValue()));
         }
-
         return ace;
     }
 
     public void visitAnnotations(AnnotatedNode node) {
-        List annotations = node.getAnnotations();
+        List<AnnotationNode> annotations = node.getAnnotations();
         if (annotations.isEmpty()) return;
-        Iterator it = annotations.iterator();
-        while (it.hasNext()) {
-            AnnotationNode an = (AnnotationNode) it.next();
+        Map<String, AnnotationNode> tmpAnnotations = new HashMap<String, AnnotationNode>();
+        ClassNode annType;
+        for (AnnotationNode an : annotations) {
             // skip built-in properties
             if (an.isBuiltIn()) continue;
-            ClassNode type = an.getClassNode();
-            resolveOrFail(type, ",  unable to find class for annotation", an);
-            for (Iterator iter = an.getMembers().entrySet().iterator(); iter.hasNext();) {
-                Map.Entry member = (Map.Entry) iter.next();
-                Expression memberValue = (Expression) member.getValue();
-                Expression newValue = transform(memberValue);
+            annType = an.getClassNode();
+            resolveOrFail(annType, ",  unable to find class for annotation", an);
+            for (Map.Entry<String, Expression> member : an.getMembers().entrySet()) {
+                Expression newValue = transform(member.getValue());
+                newValue = transformInlineConstants(newValue);
                 member.setValue(newValue);
-                if (newValue instanceof PropertyExpression) {
-                    PropertyExpression pe = (PropertyExpression) newValue;
-                    if (!(pe.getObjectExpression() instanceof ClassExpression)) {
-                        addError("unable to find class for enum",pe.getObjectExpression());
+                checkAnnotationMemberValue(newValue);
+            }
+            if(annType.isResolved()) {
+            	Class annTypeClass = annType.getTypeClass();
+            	Retention retAnn = (Retention) annTypeClass.getAnnotation(Retention.class);
+            	if (retAnn != null && retAnn.value().equals(RetentionPolicy.RUNTIME)) {
+            		AnnotationNode anyPrevAnnNode = tmpAnnotations.put(annTypeClass.getName(), an);
+            		if(anyPrevAnnNode != null) {
+            			addError("Cannot specify duplicate annotation on the same member : " + annType.getName(), an);
+            		}
+            	}
+            }
+        }
+    }
+
+    // resolve constant-looking expressions statically (do here as gets transformed away later)
+    private Expression transformInlineConstants(Expression exp) {
+        if (exp instanceof PropertyExpression) {
+            PropertyExpression pe = (PropertyExpression) exp;
+            if (pe.getObjectExpression() instanceof ClassExpression) {
+                ClassExpression ce = (ClassExpression) pe.getObjectExpression();
+                ClassNode type = ce.getType();
+                if (type.isEnum())
+                    return exp;
+
+                FieldNode fn = type.getField(pe.getPropertyAsString());
+                if (fn != null && !fn.isEnum() && fn.isStatic() && fn.isFinal()) {
+                    if (fn.getInitialValueExpression() instanceof ConstantExpression) {
+                        return fn.getInitialValueExpression();
                     }
                 }
             }
+        } else if (exp instanceof ListExpression) {
+            ListExpression le = (ListExpression) exp;
+            ListExpression result = new ListExpression();
+            for (Expression e : le.getExpressions()) {
+                result.addExpression(transformInlineConstants(e));
+            }
+            return result;
+        } else if (exp instanceof AnnotationConstantExpression) {
+            ConstantExpression ce = (ConstantExpression) exp;
+            if (ce.getValue() instanceof AnnotationNode) {
+                // replicate a little bit of AnnotationVisitor here
+                // because we can't wait until later to do this
+                AnnotationNode an = (AnnotationNode) ce.getValue();
+                for (Map.Entry<String, Expression> member : an.getMembers().entrySet()) {
+                    member.setValue(transformInlineConstants(member.getValue()));
+                }
+
+            }
+        }
+        return exp;
+    }
+
+    private void checkAnnotationMemberValue(Expression newValue) {
+        if (newValue instanceof PropertyExpression) {
+            PropertyExpression pe = (PropertyExpression) newValue;
+            if (!(pe.getObjectExpression() instanceof ClassExpression)) {
+                addError("unable to find class '" + pe.getText() + "' for annotation attribute constant", pe.getObjectExpression());
+            }
+        } else if (newValue instanceof ListExpression) {
+            ListExpression le = (ListExpression) newValue;
+            for (Expression e : le.getExpressions()) {
+                checkAnnotationMemberValue(e);
+            }
         }
     }
 
@@ -982,20 +1123,18 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         ModuleNode module = node.getModule();
         if (!module.hasImportsResolved()) {
             List l = module.getImports();
-            for (Iterator iter = l.iterator(); iter.hasNext();) {
-                ImportNode element = (ImportNode) iter.next();
-                ClassNode type = element.getType();
+            for (ImportNode importNode : module.getImports()) {
+                ClassNode type = importNode.getType();
                 if (resolve(type, false, false, true)) continue;
                 addError("unable to resolve class " + type.getName(), type);
             }
-            Map importPackages = module.getStaticImportClasses();
-            for (Iterator iter = importPackages.values().iterator(); iter.hasNext();) {
-                ClassNode type = (ClassNode) iter.next();
+            for (ImportNode importNode : module.getStaticStarImports().values()) {
+                ClassNode type = importNode.getType();
                 if (resolve(type, false, false, true)) continue;
                 // May be this type belongs in the same package as the node that is doing the
                 // static import. In that case, the package may not have been explicitly specified.
                 // Try with the node's package too. If still not found, revert to original type name.
-                if(type.getPackageName() == null && node.getPackageName() != null) {
+                if (type.getPackageName() == null && node.getPackageName() != null) {
                 	String oldTypeName = type.getName();
                 	type.setName(node.getPackageName() + "." + oldTypeName);
                 	if (resolve(type, false, false, true)) continue;
@@ -1003,13 +1142,13 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 }
                 addError("unable to resolve class " + type.getName(), type);
             }
-            for (Iterator iter = module.getStaticImportAliases().values().iterator(); iter.hasNext();) {
-                ClassNode type = (ClassNode) iter.next();
+            for (ImportNode importNode : module.getStaticImports().values()) {
+                ClassNode type = importNode.getType();
                 if (resolve(type, true, true, true)) continue;
                 addError("unable to resolve class " + type.getName(), type);
             }
-            for (Iterator iter = module.getStaticImportClasses().values().iterator(); iter.hasNext();) {
-                ClassNode type = (ClassNode) iter.next();
+            for (ImportNode importNode : module.getStaticStarImports().values()) {
+                ClassNode type = importNode.getType();
                 if (resolve(type, true, true, true)) continue;
                 addError("unable to resolve class " + type.getName(), type);
             }
@@ -1017,18 +1156,34 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         }
 
         ClassNode sn = node.getUnresolvedSuperClass();
-        if (sn != null) resolveOrFail(sn, node, true);
-
-        ClassNode[] interfaces = node.getInterfaces();
-        for (int i = 0; i < interfaces.length; i++) {
-            resolveOrFail(interfaces[i], node, true);
+        if (sn != null) {
+            resolveOrFail(sn, node, true);
+            checkCyclicInheritence(node);
+        }
 
+        for (ClassNode anInterface : node.getInterfaces()) {
+            resolveOrFail(anInterface, node, true);
         }
 
         super.visitClass(node);
 
         currentClass = oldNode;
     }
+    
+    private void checkCyclicInheritence(ClassNode node) {
+        ClassNode sn = node;
+        while(true) {
+            sn = sn.getUnresolvedSuperClass();
+            if(sn == null) break;
+
+            if(node == sn.redirect()) {
+                addError("Cyclic inheritance involving " + sn.getName() + " in class " + node.getName(), node);
+                break;
+            }
+            
+            if(sn == ClassHelper.OBJECT_TYPE) break;
+        }
+    }
 
     public void visitCatchStatement(CatchStatement cs) {
         resolveOrFail(cs.getExceptionType(), cs);
@@ -1057,34 +1212,33 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private void resolveGenericsTypes(GenericsType[] types) {
         if (types == null) return;
         currentClass.setUsingGenerics(true);
-        for (int i = 0; i < types.length; i++) {
-            resolveGenericsType(types[i]);
+        for (GenericsType type : types) {
+            resolveGenericsType(type);
         }
     }
 
     private void resolveGenericsHeader(GenericsType[] types) {
         if (types == null) return;
         currentClass.setUsingGenerics(true);
-        for (int i = 0; i < types.length; i++) {
-            ClassNode type = types[i].getType();
+        for (GenericsType type : types) {
+            ClassNode classNode = type.getType();
             String name = type.getName();
-            ClassNode[] bounds = types[i].getUpperBounds();
+            ClassNode[] bounds = type.getUpperBounds();
             if (bounds != null) {
                 boolean nameAdded = false;
-                for (int j = 0; j < bounds.length; j++) {
-                    ClassNode upperBound = bounds[j];
-                    if (!nameAdded && upperBound != null || !resolve(type)) {
-                        genericParameterNames.put(name, types[i]);
-                        types[i].setPlaceholder(true);
-                        type.setRedirect(upperBound);
+                for (ClassNode upperBound : bounds) {
+                    if (!nameAdded && upperBound != null || !resolve(classNode)) {
+                        genericParameterNames.put(name, type);
+                        type.setPlaceholder(true);
+                        classNode.setRedirect(upperBound);
                         nameAdded = true;
                     }
-                    resolveOrFail(upperBound, type);
+                    resolveOrFail(upperBound, classNode);
                 }
             } else {
-                genericParameterNames.put(name, types[i]);
-                type.setRedirect(ClassHelper.OBJECT_TYPE);
-                types[i].setPlaceholder(true);
+                genericParameterNames.put(name, type);
+                classNode.setRedirect(ClassHelper.OBJECT_TYPE);
+                type.setPlaceholder(true);
             }
         }
     }
@@ -1098,8 +1252,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         ClassNode[] bounds = genericsType.getUpperBounds();
         if (!genericParameterNames.containsKey(name)) {
             if (bounds != null) {
-                for (int j = 0; j < bounds.length; j++) {
-                    ClassNode upperBound = bounds[j];
+                for (ClassNode upperBound : bounds) {
                     resolveOrFail(upperBound, genericsType);
                     type.setRedirect(upperBound);
                     resolveGenericsTypes(upperBound.getGenericsTypes());
@@ -1110,16 +1263,15 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 resolveOrFail(type, genericsType);
             }
         } else {
-            GenericsType gt = (GenericsType) genericParameterNames.get(name);
+            GenericsType gt = genericParameterNames.get(name);
             type.setRedirect(gt.getType());
             genericsType.setPlaceholder(true);
         }
 
-
         if (genericsType.getLowerBound() != null) {
             resolveOrFail(genericsType.getLowerBound(), genericsType);
         }
         resolveGenericsTypes(type.getGenericsTypes());
-        genericsType.setResolved(true);
+        genericsType.setResolved(genericsType.getType().isResolved());
     }
 }
