@@ -39,6 +39,7 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
     private boolean inPropertyExpression;
     private Expression foundConstant;
     private Expression foundArgs;
+    private boolean inAnnotation;
 
     public StaticImportVisitor(CompilationUnit cu) {
         compilationUnit = cu;
@@ -56,6 +57,14 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
         this.currentMethod = null;
     }
 
+    @Override
+    public void visitAnnotations(AnnotatedNode node) {
+        boolean oldInAnnotation = inAnnotation;
+        inAnnotation = true;
+        super.visitAnnotations(node);
+        inAnnotation = oldInAnnotation;
+    }
+
     public Expression transform(Expression exp) {
         if (exp == null) return null;
         if (exp.getClass() == VariableExpression.class) {
@@ -80,11 +89,25 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
             }
             return result;
         }
-        if (exp.getClass() == ConstantExpression.class) {
+        if (exp instanceof ConstantExpression) {
             Expression result = exp.transformExpression(this);
             if (inPropertyExpression) {
                 foundConstant = result;
             }
+            if (inAnnotation && exp instanceof AnnotationConstantExpression) {
+                ConstantExpression ce = (ConstantExpression) result;
+                if (ce.getValue() instanceof AnnotationNode) {
+                    // replicate a little bit of AnnotationVisitor here
+                    // because we can't wait until later to do this
+                    AnnotationNode an = (AnnotationNode) ce.getValue();
+                    Map<String, Expression> attributes = an.getMembers();
+                    for (Map.Entry entry : attributes.entrySet()) {
+                        Expression attrExpr = transform((Expression) entry.getValue());
+                        entry.setValue(attrExpr);
+                    }
+
+                }
+            }
             return result;
         }
         return exp.transformExpression(this);
@@ -96,6 +119,9 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
             Expression result = findStaticFieldImportFromModule(v.getName());
             if (result != null) {
             	result.setSourcePosition(ve);
+                if (inAnnotation) {
+                    result = transformInlineConstants(result);
+                }
             	return result;
             }
             if (!inPropertyExpression || inSpecialConstructorCall) addStaticVariableError(ve);
@@ -103,14 +129,47 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
         return ve;
     }
 
+    // resolve constant-looking expressions statically (do here as gets transformed away later)
+    private Expression transformInlineConstants(Expression exp) {
+        if (exp instanceof PropertyExpression) {
+            PropertyExpression pe = (PropertyExpression) exp;
+            if (pe.getObjectExpression() instanceof ClassExpression) {
+                ClassExpression ce = (ClassExpression) pe.getObjectExpression();
+                ClassNode type = ce.getType();
+                if (type.isEnum()) return exp;
+                FieldNode fn = type.getField(pe.getPropertyAsString());
+                if (fn != null && !fn.isEnum() && fn.isStatic() && fn.isFinal()) {
+                    if (fn.getInitialValueExpression() instanceof ConstantExpression) {
+                        return fn.getInitialValueExpression();
+                    }
+                }
+            }
+        } else if (exp instanceof ListExpression) {
+            ListExpression le = (ListExpression) exp;
+            ListExpression result = new ListExpression();
+            for (Expression e : le.getExpressions()) {
+                result.addExpression(transformInlineConstants(e));
+            }
+            return result;
+        }
+
+        return exp;
+    }
+
     protected Expression transformMethodCallExpression(MethodCallExpression mce) {
         Expression args = transform(mce.getArguments());
         Expression method = transform(mce.getMethod());
         Expression object = transform(mce.getObjectExpression());
         boolean isExplicitThisOrSuper = false;
+        boolean isExplicitSuper = false;
         if (object instanceof VariableExpression) {
             VariableExpression ve = (VariableExpression) object;
             isExplicitThisOrSuper = !mce.isImplicitThis() && (ve.getName().equals("this") || ve.getName().equals("super"));
+            isExplicitSuper = ve.getName().equals("super");
+            if (isExplicitSuper && currentMethod != null && currentMethod.isStatic()) {
+                addError("'super' cannot be used in a static context, use the explicit class instead.", mce);
+                return mce;
+            }
         }
 
         if (mce.isImplicitThis() || isExplicitThisOrSuper) {
@@ -189,13 +248,12 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
             }
         }
 
-        // some this/super validation
-        boolean isExplicitThisOrSuper = false;
+        // some super validation
         if (objectExpression instanceof VariableExpression) {
             VariableExpression ve = (VariableExpression) objectExpression;
-            isExplicitThisOrSuper = !pe.isImplicitThis() && (ve.getName().equals("this") || ve.getName().equals("super"));
-            if (isExplicitThisOrSuper && currentMethod != null && currentMethod.isStatic()) {
-                addError("Non-static variable '" + ve.getName() + "' cannot be referenced from the static method " + currentMethod.getName() + ".", pe);
+            boolean isExplicitSuper = ve.getName().equals("super");
+            if (isExplicitSuper && currentMethod != null && currentMethod.isStatic()) {
+                addError("'super' cannot be used in a static context, use the explicit class instead.", pe);
                 return null;
             }
         }
@@ -227,7 +285,7 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
 
     private void addStaticVariableError(VariableExpression ve) {
         // closures are always dynamic
-        // propertiesExpressions will handle the error a bit differently
+        // propertyExpressions will handle the error a bit differently
         if (!inSpecialConstructorCall && (inClosure || !ve.isInStaticContext())) return;
         if (stillResolving) return;
         if (ve.isThisExpression() || ve.isSuperExpression()) return;
@@ -244,19 +302,15 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
     private Expression findStaticFieldImportFromModule(String name) {
         ModuleNode module = currentClass.getModule();
         if (module == null) return null;
-        Map aliases = module.getStaticImportAliases();
+        Map<String, ImportNode> importNodes = module.getStaticImports();
         stillResolving = false;
-        if (aliases.containsKey(name)) {
-            ClassNode node = (ClassNode) aliases.get(name);
-            Map fields = module.getStaticImportFields();
-            String fieldName = (String) fields.get(name);
-            Expression expression = findStaticField(node, fieldName);
+        if (importNodes.containsKey(name)) {
+            ImportNode importNode = importNodes.get(name);
+            Expression expression = findStaticField(importNode.getType(), importNode.getFieldName());
             if (expression != null) return expression;
         }
-        Map importedClasses = module.getStaticImportClasses();
-        for (Object o : importedClasses.keySet()) {
-            String className = (String) o;
-            ClassNode node = (ClassNode) importedClasses.get(className);
+        for (ImportNode importNode : module.getStaticStarImports().values()) {
+            ClassNode node = importNode.getType();
             Expression expression = findStaticField(node, name);
             if (expression != null) return expression;
         }
@@ -279,30 +333,32 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
     private Expression findStaticMethodImportFromModule(Expression method, Expression args) {
         ModuleNode module = currentClass.getModule();
         if (module == null || !(method instanceof ConstantExpression)) return null;
-        Map aliases = module.getStaticImportAliases();
+        Map<String, ImportNode> importNodes = module.getStaticImports();
         ConstantExpression ce = (ConstantExpression) method;
         Object value = ce.getValue();
         // skip non-Strings, e.g. Integer
         if (!(value instanceof String)) return null;
         final String name = (String) value;
-        if (aliases.containsKey(name)) {
-            ClassNode node = (ClassNode) aliases.get(name);
-            Map fields = module.getStaticImportFields();
-            String fieldName = (String) fields.get(name);
-            Expression expression = findStaticMethod(node, fieldName, args);
+        if (importNodes.containsKey(name)) {
+            ImportNode importNode = importNodes.get(name);
+            Expression expression = findStaticMethod(importNode.getType(), importNode.getFieldName(), args);
             if (expression != null) return expression;
         }
-        Map importPackages = module.getStaticImportClasses();
-        for (Object o : importPackages.keySet()) {
-            String className = (String) o;
-            ClassNode starImportType = null;
-            if(isEnum(currentClass) && importPackages.containsKey(currentClass.getName())) {
-            	starImportType = (ClassNode) importPackages.get(currentClass.getName());
-            } else {
-                starImportType = (ClassNode) importPackages.get(className);
-            }
+        Map<String, ImportNode> importPackages = module.getStaticStarImports();
+        ClassNode starImportType = null;
+        if(isEnum(currentClass) && importPackages.containsKey(currentClass.getName())) {
+            ImportNode importNode = importPackages.get(currentClass.getName());
+            starImportType = importNode == null ? null : importNode.getType();
             Expression expression = findStaticMethod(starImportType, name, args);
             if (expression != null) return expression;
+        } else {
+            for (Map.Entry<String, ImportNode> entry : importPackages.entrySet()) {
+                String className = entry.getKey();
+                ImportNode importNode = importPackages.get(className);
+                starImportType = importNode == null ? null : importNode.getType();
+                Expression expression = findStaticMethod(starImportType, name, args);
+                if (expression != null) return expression;
+            }
         }
         return null;
     }
@@ -322,5 +378,5 @@ public class StaticImportVisitor extends ClassCodeExpressionTransformer {
 
     private boolean isEnum(ClassNode node) {
         return (node.getModifiers() & Opcodes.ACC_ENUM) != 0;
-     }
+    }
 }
