@@ -17,11 +17,11 @@ package org.codehaus.groovy.runtime;
 
 import groovy.lang.EmptyRange;
 import groovy.lang.IntRange;
+import groovy.lang.Range;
 import org.codehaus.groovy.runtime.typehandling.DefaultTypeTransformation;
 
 import java.io.Closeable;
 import java.io.IOException;
-import java.lang.reflect.Constructor;
 import java.util.*;
 import java.util.logging.Logger;
 
@@ -33,7 +33,7 @@ public class DefaultGroovyMethodsSupport {
     private static final Logger LOG = Logger.getLogger(DefaultGroovyMethodsSupport.class.getName());
 
     // helper method for getAt and putAt
-    protected static RangeInfo subListBorders(int size, IntRange range) {
+    protected static RangeInfo subListBorders(int size, Range range) {
         int from = normaliseIndex(DefaultTypeTransformation.intUnbox(range.getFrom()), size);
         int to = normaliseIndex(DefaultTypeTransformation.intUnbox(range.getTo()), size);
         boolean reverse = range.isReverse();
@@ -158,19 +158,21 @@ public class DefaultGroovyMethodsSupport {
     }
 
     protected static <T> List<T> createSimilarList(List<T> orig, int newCapacity) {
-        if (orig instanceof LinkedList) {
+        if (orig instanceof LinkedList)
             return new LinkedList<T>();
-        } else if (orig instanceof Stack) {
+
+        if (orig instanceof Stack)
             return new Stack<T>();
-        } else if (orig instanceof Vector) {
+
+        if (orig instanceof Vector)
             return new Vector<T>();
-        }
+
         return new ArrayList<T>(newCapacity);
     }
 
     protected static <T> Set<T> createSimilarSet(Set<T> orig) {
         if (orig instanceof SortedSet) {
-            return new TreeSet<T>();
+            return new TreeSet<T>(((SortedSet)orig).comparator());
         }
         if (orig instanceof LinkedHashSet) {
             return new LinkedHashSet<T>();
@@ -180,7 +182,7 @@ public class DefaultGroovyMethodsSupport {
 
     protected static <K, V> Map<K, V> createSimilarMap(Map<K, V> orig) {
         if (orig instanceof SortedMap) {
-            return new TreeMap<K, V>();
+            return new TreeMap<K, V>(((SortedMap)orig).comparator());
         }
         if (orig instanceof Properties) {
             return (Map<K, V>) new Properties();
