@@ -16,10 +16,9 @@
 package org.codehaus.groovy.runtime;
 
 import groovy.lang.*;
-
 import org.codehaus.groovy.runtime.metaclass.MissingMethodExceptionNoStack;
-import org.codehaus.groovy.runtime.metaclass.MissingPropertyExceptionNoStack;
 import org.codehaus.groovy.runtime.metaclass.MissingMethodExecutionFailed;
+import org.codehaus.groovy.runtime.metaclass.MissingPropertyExceptionNoStack;
 import org.codehaus.groovy.runtime.typehandling.DefaultTypeTransformation;
 import org.codehaus.groovy.runtime.wrappers.GroovyObjectWrapper;
 import org.codehaus.groovy.runtime.wrappers.PojoWrapper;
@@ -33,18 +32,18 @@ import java.util.regex.Pattern;
  * A static helper class to interface bytecode and runtime
  *
  * @author Jochen Theodorou
- * @version $Revision: 11920 $
+ * @version $Revision: 17783 $
  */
 public class ScriptBytecodeAdapter {
     public static final Object[] EMPTY_ARGS = {};
-    private static final Integer ZERO = new Integer(0);
-    private static final Integer MINUS_ONE = new Integer(-1);
-    private static final Integer ONE = new Integer(1);
+    private static final Integer ZERO = Integer.valueOf(0);
+    private static final Integer MINUS_ONE = Integer.valueOf(-1);
+    private static final Integer ONE = Integer.valueOf(1);
 
     //  --------------------------------------------------------
     //                   exception handling
     //  --------------------------------------------------------
-    private static Throwable unwrap(GroovyRuntimeException gre) {
+    public static Throwable unwrap(GroovyRuntimeException gre) {
         if (gre instanceof MissingPropertyExceptionNoStack) {
             MissingPropertyExceptionNoStack noStack = (MissingPropertyExceptionNoStack) gre;
             return new MissingPropertyException(noStack.getProperty(), noStack.getType());
@@ -79,7 +78,7 @@ public class ScriptBytecodeAdapter {
                 }
             } catch (MissingMethodException e) {
                 if (e instanceof MissingMethodExecutionFailed) {
-                    throw e.getCause();
+                    throw (MissingMethodException)e.getCause();
                 } else if (!intercepting && receiver.getClass() == e.getType() && e.getMethod().equals(messageName)) {
                     // in case there's nothing else, invoke the object's own invokeMethod()
                     result = receiver.invokeMethod(messageName, messageArguments);
@@ -87,8 +86,8 @@ public class ScriptBytecodeAdapter {
                     throw e;
                 }
             }
-        } catch (GroovyRuntimeException t) {
-            throw unwrap(t);
+        } catch (GroovyRuntimeException gre) {
+            throw unwrap(gre);
         }
         return result;
     }
@@ -98,9 +97,6 @@ public class ScriptBytecodeAdapter {
     }
 
     public static Object invokeMethodOnCurrentNSpreadSafe(Class senderClass, GroovyObject receiver, String messageName, Object[] messageArguments) throws Throwable {
-        if (!isSpreadable(receiver))
-            return invokeMethodOnCurrentN(senderClass, receiver, messageName, messageArguments);
-
         List answer = new ArrayList();
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             answer.add(invokeMethodNSafe(senderClass, it.next(), messageName, messageArguments));
@@ -129,8 +125,8 @@ public class ScriptBytecodeAdapter {
         Object result = null;
         try {
             result = metaClass.invokeMethod(senderClass, receiver, messageName, messageArguments, true, true);
-        } catch (GroovyRuntimeException t) {
-            throw unwrap(t);
+        } catch (GroovyRuntimeException gre) {
+            throw unwrap(gre);
         }
         return result;
     }
@@ -140,9 +136,6 @@ public class ScriptBytecodeAdapter {
     }
 
     public static Object invokeMethodOnSuperNSpreadSafe(Class senderClass, GroovyObject receiver, String messageName, Object[] messageArguments) throws Throwable {
-        if (!isSpreadable(receiver))
-            return invokeMethodOnSuperN(senderClass, receiver, messageName, messageArguments);
-
         List answer = new ArrayList();
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             answer.add(invokeMethodNSafe(senderClass, it.next(), messageName, messageArguments));
@@ -166,7 +159,7 @@ public class ScriptBytecodeAdapter {
     //              normal method invocation
     //  --------------------------------------------------------
     public static Object invokeMethodN(Class senderClass, Object receiver, String messageName, Object[] messageArguments) throws Throwable {
-        try {
+        try {    
             return InvokerHelper.invokeMethod(receiver, messageName, messageArguments);
         } catch (GroovyRuntimeException gre) {
             throw unwrap(gre);
@@ -180,9 +173,6 @@ public class ScriptBytecodeAdapter {
 
     public static Object invokeMethodNSpreadSafe(Class senderClass, Object receiver, String messageName, Object[] messageArguments) throws Throwable {
         if (receiver == null) return null;
-        if (!isSpreadable(receiver))
-            return invokeMethodN(senderClass, receiver, messageName, messageArguments);
-
         List answer = new ArrayList();
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             answer.add(invokeMethodNSafe(senderClass, it.next(), messageName, messageArguments));
@@ -241,9 +231,13 @@ public class ScriptBytecodeAdapter {
     //       special constructor invocation (via this/super)
     //  --------------------------------------------------------
 
-    public static int selectConstructorAndTransformArguments(Object[] arguments, int numberOfConstructors, Class which) {
+    public static int selectConstructorAndTransformArguments(Object[] arguments, int numberOfConstructors, Class which) throws Throwable {
         MetaClass metaClass = GroovySystem.getMetaClassRegistry().getMetaClass(which);
-        return metaClass.selectConstructorAndTransformArguments(numberOfConstructors, arguments);
+        try {
+            return metaClass.selectConstructorAndTransformArguments(numberOfConstructors, arguments);
+        } catch (GroovyRuntimeException gre) {
+            throw unwrap(gre);
+        }
     }
 
     //  --------------------------------------------------------
@@ -268,9 +262,6 @@ public class ScriptBytecodeAdapter {
     }
 
     public static Object getFieldOnSuperSpreadSafe(Class senderClass, Object receiver, String messageName) throws Throwable {
-        if (!isSpreadable(receiver))
-            return getFieldOnSuper(senderClass, receiver, messageName);
-
         List answer = new ArrayList();
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             answer.add(getFieldOnSuper(senderClass, it.next(), messageName));
@@ -300,11 +291,6 @@ public class ScriptBytecodeAdapter {
     }
 
     public static void setFieldOnSuperSpreadSafe(Object messageArgument, Class senderClass, Object receiver, String messageName) throws Throwable {
-        if (!isSpreadable(receiver)) {
-            setFieldOnSuper(messageArgument, senderClass, receiver, messageName);
-            return;
-        }
-
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             setFieldOnSuper(messageArgument, senderClass, it.next(), messageName);
         }
@@ -319,7 +305,7 @@ public class ScriptBytecodeAdapter {
             return InvokerHelper.getAttribute(receiver, messageName);
         } catch (GroovyRuntimeException gre) {
             throw unwrap(gre);
-        }
+        }    
     }
 
     public static Object getFieldSafe(Class senderClass, Object receiver, String messageName) throws Throwable {
@@ -329,12 +315,8 @@ public class ScriptBytecodeAdapter {
 
     public static Object getFieldSpreadSafe(Class senderClass, Object receiver, String messageName) throws Throwable {
         if (receiver == null) return null;
-        if (!isSpreadable(receiver))
-            return getField(senderClass, receiver, messageName);
-
-        List list = (List) receiver;
         List answer = new ArrayList();
-        for (Iterator it = list.iterator(); it.hasNext();) {
+        for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             answer.add(getFieldSafe(senderClass, it.next(), messageName));
         }
         return answer;
@@ -359,11 +341,6 @@ public class ScriptBytecodeAdapter {
 
     public static void setFieldSpreadSafe(Object messageArgument, Class senderClass, Object receiver, String messageName) throws Throwable {
         if (receiver == null) return;
-        if (!isSpreadable(receiver)) {
-            setField(messageArgument, senderClass, receiver, messageName);
-            return;
-        }
-
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             setFieldSafe(messageArgument, senderClass, it.next(), messageName);
         }
@@ -374,18 +351,24 @@ public class ScriptBytecodeAdapter {
     //  --------------------------------------------------------
 
     public static Object getGroovyObjectField(Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
-        return receiver.getMetaClass().getAttribute(receiver, messageName);
+        try  {
+            return receiver.getMetaClass().getAttribute(receiver, messageName);
+        } catch (GroovyRuntimeException gre) {
+            throw unwrap(gre);
+        }
     }
 
     public static Object getGroovyObjectFieldSafe(Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
         if (receiver == null) return null;
-        return receiver.getMetaClass().getAttribute(receiver, messageName);
+        try {
+            return receiver.getMetaClass().getAttribute(receiver, messageName);
+        } catch (GroovyRuntimeException gre) {
+            throw unwrap(gre);
+        }
     }
 
     public static Object getGroovyObjectFieldSpreadSafe(Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
         if (receiver == null) return null;
-        if (!isSpreadable(receiver)) return getGroovyObjectField(senderClass, receiver, messageName);
-
         List answer = new ArrayList();
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             answer.add(getFieldSafe(senderClass, it.next(), messageName));
@@ -398,21 +381,24 @@ public class ScriptBytecodeAdapter {
     //  --------------------------------------------------------
 
     public static void setGroovyObjectField(Object messageArgument, Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
-        receiver.getMetaClass().setAttribute(receiver, messageName, messageArgument);
+        try {
+            receiver.getMetaClass().setAttribute(receiver, messageName, messageArgument);
+        } catch (GroovyRuntimeException gre) {
+            throw unwrap(gre);
+        }
     }
 
     public static void setGroovyObjectFieldSafe(Object messageArgument, Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
         if (receiver == null) return;
-        receiver.getMetaClass().setAttribute(receiver, messageName, messageArgument);
+        try {
+            receiver.getMetaClass().setAttribute(receiver, messageName, messageArgument);
+        } catch (GroovyRuntimeException gre) {
+            throw unwrap(gre);
+        }
     }
 
     public static void setGroovyObjectFieldSpreadSafe(Object messageArgument, Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
         if (receiver == null) return;
-        if (!isSpreadable(receiver)) {
-            setGroovyObjectField(messageArgument, senderClass, receiver, messageName);
-            return;
-        }
-
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             setFieldSafe(messageArgument, senderClass, it.next(), messageName);
         }
@@ -431,9 +417,6 @@ public class ScriptBytecodeAdapter {
     }
 
     public static Object getPropertyOnSuperSpreadSafe(Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
-        if (!isSpreadable(receiver))
-            return getPropertyOnSuper(senderClass, receiver, messageName);
-
         List answer = new ArrayList();
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             answer.add(getPropertySafe(senderClass, it.next(), messageName));
@@ -458,11 +441,6 @@ public class ScriptBytecodeAdapter {
     }
 
     public static void setPropertyOnSuperSpreadSafe(Object messageArgument, Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
-        if (!isSpreadable(receiver)) {
-            setPropertyOnSuper(messageArgument, senderClass, receiver, messageName);
-            return;
-        }
-
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             setPropertySafe(messageArgument, senderClass, it.next(), messageName);
         }
@@ -487,8 +465,6 @@ public class ScriptBytecodeAdapter {
 
     public static Object getPropertySpreadSafe(Class senderClass, Object receiver, String messageName) throws Throwable {
         if (receiver == null) return null;
-        if (!isSpreadable(receiver))
-            return getProperty(senderClass, receiver, messageName);
 
         List answer = new ArrayList();
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
@@ -503,6 +479,7 @@ public class ScriptBytecodeAdapter {
 
     public static void setProperty(Object messageArgument, Class senderClass, Object receiver, String messageName) throws Throwable {
         try {
+            if (receiver==null) receiver=NullObject.getNullObject();
             InvokerHelper.setProperty(receiver, messageName, messageArgument);
         } catch (GroovyRuntimeException gre) {
             throw unwrap(gre);
@@ -516,10 +493,6 @@ public class ScriptBytecodeAdapter {
 
     public static void setPropertySpreadSafe(Object messageArgument, Class senderClass, Object receiver, String messageName) throws Throwable {
         if (receiver == null) return;
-        if (!isSpreadable(receiver)) {
-            setProperty(messageArgument, senderClass, receiver, messageName);
-            return;
-        }
 
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             setPropertySafe(messageArgument, senderClass, it.next(), messageName);
@@ -531,11 +504,7 @@ public class ScriptBytecodeAdapter {
     //  --------------------------------------------------------
 
     public static Object getGroovyObjectProperty(Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
-        try {
-            return receiver.getProperty(messageName);
-        } catch (GroovyRuntimeException gre) {
-            throw unwrap(gre);
-        }
+        return receiver.getProperty(messageName);
     }
 
     public static Object getGroovyObjectPropertySafe(Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
@@ -545,7 +514,6 @@ public class ScriptBytecodeAdapter {
 
     public static Object getGroovyObjectPropertySpreadSafe(Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
         if (receiver == null) return null;
-        if (!isSpreadable(receiver)) return getGroovyObjectProperty(senderClass, receiver, messageName);
 
         List answer = new ArrayList();
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
@@ -559,7 +527,11 @@ public class ScriptBytecodeAdapter {
     //  --------------------------------------------------------
 
     public static void setGroovyObjectProperty(Object messageArgument, Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
-        receiver.setProperty(messageName, messageArgument);
+        try {
+            receiver.setProperty(messageName, messageArgument);
+        } catch (GroovyRuntimeException gre) {
+            throw unwrap(gre);
+        }
     }
 
     public static void setGroovyObjectPropertySafe(Object messageArgument, Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
@@ -569,10 +541,6 @@ public class ScriptBytecodeAdapter {
 
     public static void setGroovyObjectPropertySpreadSafe(Object messageArgument, Class senderClass, GroovyObject receiver, String messageName) throws Throwable {
         if (receiver == null) return;
-        if (!isSpreadable(receiver)) {
-            setProperty(messageArgument, senderClass, receiver, messageName);
-            return;
-        }
 
         for (Iterator it = InvokerHelper.asIterator(receiver); it.hasNext();) {
             setPropertySafe(messageArgument, senderClass, it.next(), messageName);
@@ -631,11 +599,7 @@ public class ScriptBytecodeAdapter {
      * @throws Throwable if the type casting fails
      */
     public static Object castToType(Object object, Class type) throws Throwable {
-        try {
             return DefaultTypeTransformation.castToType(object, type);
-        } catch (GroovyRuntimeException gre) {
-            throw unwrap(gre);
-        }
     }
 
     public static Tuple createTuple(Object[] array) {
@@ -661,6 +625,21 @@ public class ScriptBytecodeAdapter {
 
     //TODO: refactor
     public static List createRange(Object from, Object to, boolean inclusive) throws Throwable {
+        if (from instanceof Integer && to instanceof Integer) {
+            int ito = (Integer) to;
+            int ifrom = (Integer) from;
+            if (!inclusive) {
+                if (ifrom == ito) {
+                    return new EmptyRange((Comparable) from);
+                }
+                if (ifrom > ito) {
+                    ito++;
+                } else {
+                    ito--;
+                }
+            }
+            return new IntRange(ifrom, ito);
+        }
         if (!inclusive) {
             if (compareEqual(from, to)) {
                 return new EmptyRange((Comparable) from);
@@ -671,11 +650,11 @@ public class ScriptBytecodeAdapter {
                 to = invokeMethod0(ScriptBytecodeAdapter.class, to, "previous");
             }
         }
-        if (from instanceof Integer && to instanceof Integer) {
+
+        if (from instanceof Integer && to instanceof Integer)
             return new IntRange(DefaultTypeTransformation.intUnbox(from), DefaultTypeTransformation.intUnbox(to));
-        } else {
+        else
             return new ObjectRange((Comparable) from, (Comparable) to);
-        }
     }
 
     //assert
@@ -736,11 +715,7 @@ public class ScriptBytecodeAdapter {
     }
 
     public static Matcher findRegex(Object left, Object right) throws Throwable {
-        try {
             return InvokerHelper.findRegex(left, right);
-        } catch (GroovyRuntimeException gre) {
-            throw unwrap(gre);
-        }
     }
 
     public static boolean matchRegex(Object left, Object right) {
@@ -780,29 +755,26 @@ public class ScriptBytecodeAdapter {
     }
 
     public static Object unaryMinus(Object value) throws Throwable {
-        try {
             return InvokerHelper.unaryMinus(value);
+    }
+
+    public static Object unaryPlus(Object value) throws Throwable {
+        try {
+            return InvokerHelper.unaryPlus(value);
         } catch (GroovyRuntimeException gre) {
             throw unwrap(gre);
         }
     }
 
-    public static Object unaryPlus(Object value) {
-        return InvokerHelper.unaryPlus(value);
-    }
-
-    public static Object bitwiseNegate(Object value) {
-        return InvokerHelper.bitwiseNegate(value);
+    public static Object bitwiseNegate(Object value) throws Throwable {
+        try {
+            return InvokerHelper.bitwiseNegate(value);
+        } catch (GroovyRuntimeException gre) {
+            throw unwrap(gre);
+        }
     }
 
     public static MetaClass initMetaClass(Object object) {
-        return InvokerHelper.getMetaClass(object);
-    }
-
-    private static boolean isSpreadable (Object receiver) {
-        return (receiver instanceof Collection)
-          || (receiver instanceof Map)
-          || (receiver instanceof Iterator)
-          || receiver.getClass().isArray();
+        return InvokerHelper.getMetaClass(object.getClass());
     }
 }
