@@ -11,10 +11,10 @@ public final class NullCallSite extends AbstractCallSite {
         super(callSite);
     }
 
-    public final Object call(Object receiver, Object[] args) throws Throwable {
+    public Object call(Object receiver, Object[] args) throws Throwable {
         if (receiver == null) {
             try{
-                return InvokerHelper.invokeMethod(NullObject.getNullObject(), name, args);
+                return CallSiteArray.defaultCall(this, NullObject.getNullObject(), args);
             } catch (GroovyRuntimeException gre) {
                 throw ScriptBytecodeAdapter.unwrap(gre);
             }
