@@ -25,12 +25,12 @@ import org.codehaus.groovy.ast.ASTNode;
  * Should be replaced when no longer required.
  *
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
- * @version $Revision: 7922 $
+ * @version $Revision: 17491 $
  */
 public class RuntimeParserException extends GroovyRuntimeException {
 
     public RuntimeParserException(String message, ASTNode node) {
-        super(message + ".\nNode: " + node.getClass().getName(), node);
+        super(message + "\n", node);
     }
 
     public void throwParserException() throws SyntaxException {
