@@ -27,6 +27,7 @@ import org.codehaus.groovy.ast.stmt.ExpressionStatement;
 import org.codehaus.groovy.ast.stmt.Statement;
 import org.codehaus.groovy.classgen.Verifier;
 import org.codehaus.groovy.control.ResolveVisitor;
+import org.codehaus.groovy.tools.Utilities;
 import org.objectweb.asm.Opcodes;
 
 import java.io.*;
@@ -44,6 +45,7 @@ public class JavaStubGenerator
         this.outputPath = outputPath;
         this.requireSuperResolved = requireSuperResolved;
         this.java5 = java5;
+        outputPath.mkdirs();
     }
 
     public JavaStubGenerator(final File outputPath) {
@@ -63,6 +65,13 @@ public class JavaStubGenerator
             return;
         }
 
+        // owner should take care for us
+        if (classNode instanceof InnerClassNode)
+            return;
+
+        // don't generate stubs for private classes, as they are only visible in the same file
+        if ((classNode.getModifiers() & Opcodes.ACC_PRIVATE) != 0) return;
+
         String fileName = classNode.getName().replace('.', '/');
         mkdirs(outputPath,fileName);
         toCompile.add(fileName);
@@ -71,17 +80,6 @@ public class JavaStubGenerator
         FileOutputStream fos = new FileOutputStream(file);
         PrintWriter out = new PrintWriter(fos);
         
-        Verifier verifier = new Verifier() {
-            public void addCovariantMethods(ClassNode cn) {}
-            protected void addTimeStamp(ClassNode node) {}
-            protected void addInitialization(ClassNode node) {}   
-            protected void addPropertyMethod(MethodNode method) {
-            	propertyMethods.add(method);
-            }
-            protected void addReturnIfNeeded(MethodNode node) {}
-        };
-        verifier.visitClass(classNode);
-        
         try {
             String packageName = classNode.getPackageName();
             if (packageName != null) {
@@ -90,11 +88,40 @@ public class JavaStubGenerator
 
             genImports(classNode, out);
 
+            genClassInner(classNode, out);
+
+        } finally {
+            try {
+                out.close();
+            } catch (Exception e) {
+                // ignore
+            }
+            try {
+                fos.close();
+            } catch (IOException e) {
+                // ignore
+            }
+        }
+    }
+
+    private void genClassInner(ClassNode classNode, PrintWriter out) throws FileNotFoundException {
+        try {
+            Verifier verifier = new Verifier() {
+                public void addCovariantMethods(ClassNode cn) {}
+                protected void addTimeStamp(ClassNode node) {}
+                protected void addInitialization(ClassNode node) {}
+                protected void addPropertyMethod(MethodNode method) {
+                    propertyMethods.add(method);
+                }
+                protected void addReturnIfNeeded(MethodNode node) {}
+            };
+            verifier.visitClass(classNode);
+
             boolean isInterface = classNode.isInterface();
             boolean isEnum = (classNode.getModifiers() & Opcodes.ACC_ENUM) !=0;
             printModifiers(out, classNode.getModifiers()
                     & ~(isInterface ? Opcodes.ACC_ABSTRACT : 0));
-            
+
             if (isInterface) {
                 out.print ("interface ");
             } else if (isEnum) {
@@ -102,7 +129,11 @@ public class JavaStubGenerator
             } else {
                 out.print ("class ");
             }
-            out.println(classNode.getNameWithoutPackage());
+
+            String className = classNode.getNameWithoutPackage();
+            if (classNode instanceof InnerClassNode)
+                className = className.substring(className.lastIndexOf("$")+1);
+            out.println(className);
             writeGenericsBounds(out, classNode, true);
 
             ClassNode superClass = classNode.getUnresolvedSuperClass(false);
@@ -110,7 +141,7 @@ public class JavaStubGenerator
             if (!isInterface && !isEnum) {
                 out.print("  extends ");
                 printType(superClass,out);
-            } 
+            }
 
             ClassNode[] interfaces = classNode.getInterfaces();
             if (interfaces != null && interfaces.length > 0) {
@@ -132,19 +163,14 @@ public class JavaStubGenerator
             genFields(classNode, out, isEnum);
             genMethods(classNode, out, isEnum);
 
+            for (Iterator<InnerClassNode> inner = classNode.getInnerClasses(); inner.hasNext(); ) {
+                genClassInner(inner.next(), out);
+            }
+
             out.println("}");
-        } finally {
-        	propertyMethods.clear();
-            try {
-                out.close();
-            } catch (Exception e) {
-                // ignore
-            }
-            try {
-                fos.close();
-            } catch (IOException e) {
-                // ignore
-            }
+        }
+        finally {
+            propertyMethods.clear();
         }
     }
 
@@ -256,7 +282,10 @@ public class JavaStubGenerator
         // printModifiers(out, constructorNode.getModifiers());
 
         out.print("public "); // temporary hack
-        out.print(clazz.getNameWithoutPackage());
+        String className = clazz.getNameWithoutPackage();
+        if (clazz instanceof InnerClassNode)
+            className = className.substring(className.lastIndexOf("$")+1);
+        out.println(className);
 
         printParams(constructorNode, out);
 
@@ -367,7 +396,8 @@ public class JavaStubGenerator
 
     private void genMethod(ClassNode clazz, MethodNode methodNode, PrintWriter out) {
         if (methodNode.getName().equals("<clinit>")) return;
-        
+        if (methodNode.isPrivate() || !Utilities.isJavaIdentifier(methodNode.getName())) return;
+
         if (!clazz.isInterface()) printModifiers(out, methodNode.getModifiers());
 
         writeGenericsBounds(out, methodNode.getGenericsTypes());
@@ -546,7 +576,7 @@ public class JavaStubGenerator
     }
 
     private void genImports(ClassNode classNode, PrintWriter out) {
-        Set imports = new HashSet();
+        Set<String> imports = new HashSet<String>();
 
         //
         // HACK: Add the default imports... since things like Closure and GroovyObject seem to parse out w/o fully qualified classnames.
@@ -554,20 +584,18 @@ public class JavaStubGenerator
         imports.addAll(Arrays.asList(ResolveVisitor.DEFAULT_IMPORTS));
         
         ModuleNode moduleNode = classNode.getModule();
-        for (Iterator it = moduleNode.getImportPackages().iterator(); it.hasNext();) {
-            imports.add(it.next());
+        for (ImportNode importNode : moduleNode.getStarImports()) {
+            imports.add(importNode.getPackageName());
         }
 
-        for (Iterator it = moduleNode.getImports().iterator(); it.hasNext();) {
-            ImportNode imp = (ImportNode) it.next();
+        for (ImportNode imp : moduleNode.getImports()) {
             String name = imp.getType().getName();
             int lastDot = name.lastIndexOf('.');
             if (lastDot != -1)
                 imports.add(name.substring(0, lastDot + 1));
         }
 
-        for (Iterator it = imports.iterator(); it.hasNext();) {
-            String imp = (String) it.next();
+        for (String imp : imports) {
             out.print("import ");
             out.print(imp);
             out.println("*;");
