@@ -32,7 +32,7 @@ import jline.Terminal;
 /**
  * Provides support for using ANSI color escape codes.
  *
- * @version $Id: ANSI.java 11949 2008-04-22 09:19:18Z paulk $
+ * @version $Id: ANSI.java 17536 2009-09-04 01:42:05Z glaforge $
  * @author <a href="mailto:jason@planet57.com">Jason Dillon</a>
  */
 public class ANSI
@@ -257,7 +257,7 @@ public class ANSI
 
         private static final int BEGIN_TOKEN_SIZE = BEGIN_TOKEN.length();
 
-        public static final String END_TOKEN = "|";
+        public static final String END_TOKEN = "|@";
 
         private static final int END_TOKEN_SIZE = END_TOKEN.length();
 
