@@ -1,5 +1,5 @@
 /*
- * $Id: GroovyShellTest.java 15261 2009-02-05 12:18:38Z paulk $version
+ * $Id: GroovyShellTest.java 17820 2009-09-30 22:37:55Z blackdrag $version
  * Nov 23, 2003 9:02:55 PM $user Exp $
  * 
  * Copyright 2003 (C) Sam Pullara. All Rights Reserved.
@@ -106,4 +106,17 @@ public class GroovyShellTest extends GroovyTestCase {
         assertEquals("I like Cheddar", result);
         assertEquals("abc", context.getVariable("x"));
     }
+    
+    public void testClassLoader() {
+        Binding context = new Binding();
+        CompilerConfiguration config = new CompilerConfiguration();
+        config.setScriptBaseClass(DerivedScript.class.getName());
+        GroovyShell shell = new GroovyShell(context, config);
+        String script = "evaluate '''\n"+
+                        "class XXXX{}\n"+
+                        "assert evaluate('XXXX') == XXXX\n"+
+                        "'''";
+        shell.evaluate(script);
+     
+    }
 }
