@@ -1,5 +1,5 @@
 /*
- $Id: DefaultGroovyMethodsTest.java 17057 2009-07-25 23:48:31Z paulk $
+ $Id: DefaultGroovyMethodsTest.java 17058 2009-07-26 00:18:28Z paulk $
 
  Copyright 2003 (C) James Strachan and Bob Mcwhirter. All Rights Reserved.
 
@@ -66,10 +66,10 @@ import java.util.Map;
 public class DefaultGroovyMethodsTest extends GroovyTestCase {
 
     public void testPrint() throws Exception {
-        Map map = new HashMap();
+        Map<String, String> map = new HashMap<String, String>();
         map.put("bob", "drools");
         map.put("james", "geronimo");
-        List list = new ArrayList();
+        List<Map<String, String>> list = new ArrayList<Map<String, String>>();
         list.add(map);
         assertEquals("[[james:geronimo, bob:drools]]", InvokerHelper.toString(list));
     }
@@ -89,77 +89,61 @@ public class DefaultGroovyMethodsTest extends GroovyTestCase {
         assertEquals("`", answer);
         assertTrue(ScriptBytecodeAdapter.compareLessThan(answer, original));
     }
-
+    
     public void testFloatRounding() throws Exception {
-        Float f = new Float(1000.123456);
+        Float f = 1000.123456f;
 
-        assertEquals(DefaultGroovyMethods.round(f), new Float(1000.0).intValue());
-        assertEquals(round(f, 0), new Float(1000.0));
-        assertEquals(round(f, 1), new Float(1000.1));
-        assertEquals(round(f, 2), new Float(1000.12));
-        assertEquals(round(f, 3), new Float(1000.123));
-        assertEquals(round(f, 4), new Float(1000.1235));
-        assertEquals(round(f, 5), new Float(1000.12346));
-        assertEquals(round(f, 6), new Float(1000.123456));
+        assertEquals(DefaultGroovyMethods.round(f), 1000);
+        assertEquals(DefaultGroovyMethods.round(f, 0), 1000.0f);
+        assertEquals(DefaultGroovyMethods.round(f, 1), 1000.1f);
+        assertEquals(DefaultGroovyMethods.round(f, 2), 1000.12f);
+        assertEquals(DefaultGroovyMethods.round(f, 3), 1000.123f);
+        assertEquals(DefaultGroovyMethods.round(f, 4), 1000.1235f);
+        assertEquals(DefaultGroovyMethods.round(f, 5), 1000.12346f);
+        assertEquals(DefaultGroovyMethods.round(f, 6), 1000.123456f);
     }
 
     public void testDoubleRounding() throws Exception {
-        Double d = new Double(1000.123456);
+        Double d = 1000.123456;
 
-        assertEquals(DefaultGroovyMethods.round(d), new Double(1000.0).longValue());
-        assertEquals(round(d, 0), new Double(1000.0));
-        assertEquals(round(d, 1), new Double(1000.1));
-        assertEquals(round(d, 2), new Double(1000.12));
-        assertEquals(round(d, 3), new Double(1000.123));
-        assertEquals(round(d, 4), new Double(1000.1235));
-        assertEquals(round(d, 5), new Double(1000.12346));
-        assertEquals(round(d, 6), new Double(1000.123456));
+        assertEquals(DefaultGroovyMethods.round(d), 1000L);
+        assertEquals(DefaultGroovyMethods.round(d, 0), 1000.0);
+        assertEquals(DefaultGroovyMethods.round(d, 1), 1000.1);
+        assertEquals(DefaultGroovyMethods.round(d, 2), 1000.12);
+        assertEquals(DefaultGroovyMethods.round(d, 3), 1000.123);
+        assertEquals(DefaultGroovyMethods.round(d, 4), 1000.1235);
+        assertEquals(DefaultGroovyMethods.round(d, 5), 1000.12346);
+        assertEquals(DefaultGroovyMethods.round(d, 6), 1000.123456);
     }
 
     public void testFloatTruncate() throws Exception {
-        Float f = new Float(1000.123456);
+        Float f = 1000.123456f;
 
-        assertEquals(new Float(DefaultGroovyMethods.trunc(f)), new Float(1000.0));
-        assertEquals(trunc(f, 0), new Float(1000.0));
-        assertEquals(trunc(f, 1), new Float(1000.1));
-        assertEquals(trunc(f, 2), new Float(1000.12));
-        assertEquals(trunc(f, 3), new Float(1000.123));
-        assertEquals(trunc(f, 4), new Float(1000.1234));
-        assertEquals(trunc(f, 5), new Float(1000.12345));
-        assertEquals(trunc(f, 6), new Float(1000.123456));
+        assertEquals(DefaultGroovyMethods.trunc(f), 1000.0f);
+        assertEquals(DefaultGroovyMethods.trunc(f, 0), 1000.0f);
+        assertEquals(DefaultGroovyMethods.trunc(f, 1), 1000.1f);
+        assertEquals(DefaultGroovyMethods.trunc(f, 2), 1000.12f);
+        assertEquals(DefaultGroovyMethods.trunc(f, 3), 1000.123f);
+        assertEquals(DefaultGroovyMethods.trunc(f, 4), 1000.1234f);
+        assertEquals(DefaultGroovyMethods.trunc(f, 5), 1000.12345f);
+        assertEquals(DefaultGroovyMethods.trunc(f, 6), 1000.123456f);
     }
 
     public void testDoubleTruncate() throws Exception {
-        Double d = new Double(1000.123456);
+        Double d = 1000.123456;
 
-        assertEquals(new Double(DefaultGroovyMethods.trunc(d)), new Double(1000.0));
-        assertEquals(trunc(d, 0), new Double(1000.0));
-        assertEquals(trunc(d, 1), new Double(1000.1));
-        assertEquals(trunc(d, 2), new Double(1000.12));
-        assertEquals(trunc(d, 3), new Double(1000.123));
-        assertEquals(trunc(d, 4), new Double(1000.1234));
-        assertEquals(trunc(d, 5), new Double(1000.12345));
-        assertEquals(trunc(d, 6), new Double(1000.123456));
-    }
-
-    private Float round(Float f, int precision) {
-        return new Float(DefaultGroovyMethods.round(f, precision));
-    }
-
-    private Double round(Double d, int precision) {
-        return new Double(DefaultGroovyMethods.round(d, precision));
-    }
-
-    private Float trunc(Float f, int precision) {
-        return new Float(DefaultGroovyMethods.trunc(f, precision));
-    }
-
-    private Double trunc(Double d, int precision) {
-        return new Double(DefaultGroovyMethods.trunc(d, precision));
+        assertEquals(DefaultGroovyMethods.trunc(d), 1000.0);
+        assertEquals(DefaultGroovyMethods.trunc(d, 0), 1000.0);
+        assertEquals(DefaultGroovyMethods.trunc(d, 1), 1000.1);
+        assertEquals(DefaultGroovyMethods.trunc(d, 2), 1000.12);
+        assertEquals(DefaultGroovyMethods.trunc(d, 3), 1000.123);
+        assertEquals(DefaultGroovyMethods.trunc(d, 4), 1000.1234);
+        assertEquals(DefaultGroovyMethods.trunc(d, 5), 1000.12345);
+        assertEquals(DefaultGroovyMethods.trunc(d, 6), 1000.123456);
     }
 
     public void testToMethods() throws Exception {
-        Number n = new Long(7);
+        Number n = 7L;
 
         assertEquals(DefaultGroovyMethods.toInteger("1"), new Integer(1));
         assertEquals(DefaultGroovyMethods.toInteger(n), new Integer(7));
@@ -177,7 +161,7 @@ public class DefaultGroovyMethodsTest extends GroovyTestCase {
         assertEquals(DefaultGroovyMethods.toBigDecimal(n), new BigDecimal("7"));
 
         // The following is true starting with 1.6 (GROOVY-3171):
-        assertEquals(new BigDecimal("0.1"), DefaultGroovyMethods.toBigDecimal(new Double(0.1d)));
+        assertEquals(new BigDecimal("0.1"), DefaultGroovyMethods.toBigDecimal(0.1));
 
         assertEquals(DefaultGroovyMethods.toURL("http://example.org/"), new URL("http://example.org/"));
         assertEquals(DefaultGroovyMethods.toURI("http://example.org/"), new URI("http://example.org/"));
